/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.ui;

import com.oranda.pacdasher.uimodel.util.Direction;
import com.oranda.pacdasher.uimodel.util.DirectionCode;
import com.oranda.pacdasher.State;
import com.oranda.pacdasher.uimodel.Maze;
import com.oranda.pacdasher.uimodel.PacDasher;
import com.oranda.pacdasher.uimodel.UIModel;
import com.oranda.util.Output;

import java.awt.Canvas;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;

/*
 * Contains the main moveAndDraw method which is called in a loop.
 * Also responds to events from the user to change the State
 */
public class Controller implements KeyListener
{
    // the one true instance
    private static Controller controller;
    
    protected PacCanvas pacCanvas;
    protected InfoCanvas infoCanvas;
    protected LivesCanvas livesCanvas;
    
    protected static MazeRenderer mazeRenderer;
            
    private Controller() {}
        
    public void initialize()
    {
        this.pacCanvas = GUI.getInstance().getPacCanvas();
        this.infoCanvas = GUI.getInstance().getInfoCanvas();
        this.livesCanvas = GUI.getInstance().getLivesCanvas();
        
        mazeRenderer = new MazeRenderer(pacCanvas);
        PacDasher pacDasher = UIModel.getInstance().getPacDasher();
        pacDasher.addPacListener(infoCanvas);
        pacDasher.addPacListener(livesCanvas);
        pacCanvas.repaint();            
    }
    
    public void moveAndDraw()
    {    
       if (getState() == State.NORMAL_STATE)
        {
            UIModel.getInstance().move();
            checkGameOver();
        }
        
        pacCanvas.repaint();        
        
    }
    
    public void checkGameOver()
    {
        if (UIModel.getInstance().isGameOver())
        {
            setState(State.GAME_OVER_STATE);
        }
    }
    


    
    public static Controller getInstance()
    {
        if (controller == null) 
        {
            Output.print(Output.DEBUG, "new Controller()");
            controller = new Controller();
        }
        return controller;
    }
    
    public static void paintMaze(Graphics g)
    {
        if (getState() != State.START_STATE)
        {
            Maze maze = UIModel.getInstance().getMaze();
            mazeRenderer.paintMaze(g, maze);
            State.setRepaintNecessary(false);
        }
    }
    
    public static void paintMazeUpdate(Graphics g)
    {
        if (getState() != State.START_STATE)
        {
            Maze maze = UIModel.getInstance().getMaze();
            if (State.isRepaintNecessary())
            {
                mazeRenderer.paintMaze(g, maze);
                State.setRepaintNecessary(false);
            } 
            else
            {
                mazeRenderer.paintMazeUpdate(g, maze);
            }            
        }

    }
    
    
    public void keyPressed(KeyEvent e) 
    {
        if (getState() == State.GAME_OVER_STATE)
        {
            setState(State.START_STATE);
            UIModel.getInstance().resetVirtualTime();
        } 
        else
        {
            DirectionCode dirCode = Direction.getDirectionCode(e.getKeyCode());
            UIModel.getInstance().setDirectionCode(dirCode);
        }        
    }
    
    public void keyReleased(KeyEvent e) {}
    
    public void keyTyped(KeyEvent e) {}
    
    public static int getState()
    {
        return State.getState();
    }
    
    public static void setState(int state)
    {
        State.setState(state);
    }
}
