/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.ui;

import com.oranda.pacdasher.uimodel.MobileVisualObject;
import com.oranda.pacdasher.uimodel.util.XY;
import com.oranda.pacdasher.uimodel.util.XYCoarse;
import com.oranda.pacdasher.uimodel.util.XYCManager;
 
import java.util.Collections;
import java.util.List;
import java.util.LinkedList;

/*
 * Encapsulate the implementation of a dirty area
 * Currently tile-based.
 * This is the queue implementation - too slow
 */ 
public class DirtyAreaQueue
{

     private LinkedList queueTiles = new LinkedList();
     private final static int QUEUE_SIZE = 8;
     
     private DirtyAreaQueue()
     {
     }    
     
     public DirtyAreaQueue(XYCoarse xyCoarse)
     {
     	 // initialize 
     	 for (int i=0; i<QUEUE_SIZE; i++)
     	 {
         	queueTiles.add(xyCoarse);
         }
     }    
     
     /* 
      * Dirty the area around a Mobile Visual Object
      */
     public void addMVOArea(MobileVisualObject mvo)
     {
         XYCoarse xyCoarse = mvo.getXYCoarse();
         int xCoarse = xyCoarse.getX();
         int yCoarse = xyCoarse.getY();
         XYCManager xycm = XYCManager.getInstance();
         add(xycm.createXYC(xCoarse - 1, yCoarse - 1));
         add(xycm.createXYC(xCoarse, yCoarse - 1));
         add(xycm.createXYC(xCoarse - 1, yCoarse));
         add(xycm.createXYC(xCoarse, yCoarse));
         /*add(xycm.createXYC(xCoarse + 1, yCoarse);
         add(xycm.createXYC(xCoarse + 1, yCoarse - 1);
         add(xycm.createXYC(xCoarse - 1, yCoarse + 1);
         add(xycm.createXYC(xCoarse, yCoarse);
         */
     }
     
     
     
     public void add(XYCoarse xyCoarse)
     {
         if (!queueTiles.contains(xyCoarse))
         {
             queueTiles.addFirst(xyCoarse);
             queueTiles.removeLast();
         }
     }
     
     public List getDirtyTiles()
     {
         return queueTiles;
     }
     
     /*
     public void set(XYCoarse xycLeftTop, XYCoarse xycRightBottom)
     {
         this.xycLeftTop = xycLeftTop;
         this.xycRightBottom = xycRightBottom;
     }
     
     public XYCoarse getXYCLeftTop()
     {
         return xycLeftTop;
     }    
     
     public XYCoarse getXYCRightBottom()
     {
         return xycRightBottom;
     }
     
     public String toString()
     {
         return xycLeftTop + " to " + xycRightBottom;
     }
     
     */
     
     public String toString()
     {
     	 StringBuffer sb = new StringBuffer("");
     	 for (int i=0; i<QUEUE_SIZE; i++)
     	 {
         	XYCoarse xyCoarse = (XYCoarse) queueTiles.get(i);
         	sb.append(" ");
         	sb.append(xyCoarse);
         }
         return sb.toString();
     }
     
 }     
     
         