/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.ui;

import com.oranda.pacdasher.uimodel.util.UIModelConsts;
import com.oranda.util.Output;

import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.GraphicsDevice;
import java.awt.GraphicsConfiguration;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

/**
 * GUI - the main Frame: containing 3 parts: the score canvas
 * at the top, the main pacCanvas taking up most of the area,
 * and a little canvas showing lives at the bottom.
 */ 
public class GUI
{
    // the one true instance
    protected static GUI gui;

    protected static Frame pacFrame;
    protected static PacCanvas pacCanvas;
    protected static LivesCanvas livesCanvas;
    protected static InfoCanvas infoCanvas;
        
    /*
     * Constructor is private because it's a singleton
     */
    private GUI()
    {    
        GraphicsEnvironment graphicsEnvironment
            = GraphicsEnvironment.getLocalGraphicsEnvironment();
        GraphicsDevice graphicsDevice 
            = graphicsEnvironment.getDefaultScreenDevice();
        GraphicsConfiguration graphicsConfiguration
            = graphicsDevice.getDefaultConfiguration();        
        
        pacFrame = new Frame("PacDasher!", graphicsConfiguration);
        
        int mazeWidth =
            (UIModelConsts.MAZE_WIDTH + 1) * UIModelConsts.X_TILE_SIZE; 
        int mazeHeight = 
            (UIModelConsts.MAZE_HEIGHT + 1) * UIModelConsts.Y_TILE_SIZE + 5;

        // END create UI model

        //Output.print(Output.DEBUG, "width " + width);
        //Output.print(Output.DEBUG, "height " + height);                        

        pacCanvas = new PacCanvas();
        pacCanvas.setSize(mazeWidth, mazeHeight);    

        infoCanvas = new InfoCanvas();
        int infoWidth = mazeWidth;
        int infoHeight = (5 * UIModelConsts.Y_TILE_SIZE)/2; 
        infoCanvas.setSize(infoWidth, infoHeight);
        
        livesCanvas = new LivesCanvas();
        int livesWidth = mazeWidth;
        int livesHeight = (3 * UIModelConsts.Y_TILE_SIZE)/2;
        livesCanvas.setSize(livesWidth, livesHeight);
        

        
        pacFrame.setSize(pacCanvas.getWidth() 
            + UIModelConsts.FRAME_EDGE_SIZE * 2, 
            pacCanvas.getHeight() + infoCanvas.getHeight() 
            + livesCanvas.getHeight() 
            + UIModelConsts.FRAME_BAR_SIZE);
        pacFrame.setResizable(false);            
        pacFrame.setLocation(10,0);    
        pacFrame.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
        pacFrame.add(infoCanvas);
        pacFrame.add(pacCanvas);
        pacFrame.add(livesCanvas);
        
        pacFrame.setVisible(true);

        pacFrame.addWindowListener(new WindowAdapter() 
            {
                public void windowClosing(WindowEvent e) {System.exit(0);}
                public void windowDeiconified(WindowEvent e) {/*  sleep();*/ }
                public void windowIconified(WindowEvent e) { /* restart(); */ } 
            } 
        );                    
    }

    public void addKeyListener(KeyListener keyListener)
    {
        pacFrame.addKeyListener(keyListener);
    }
    
    public static GUI getInstance()
    {
        if (GUI.gui == null) 
        {
            GUI.gui = new GUI();
        }
        return gui;
    }

    public PacCanvas getPacCanvas()
    {
        return this.pacCanvas;
    }

    public InfoCanvas getInfoCanvas()
    {
        return this.infoCanvas;
    }

    public LivesCanvas getLivesCanvas()
    {
        return this.livesCanvas;
    }
    
    public static void paintMaze(Graphics g)
    {
        Controller.paintMaze(g);
    }
    
    public static void paintMazeUpdate(Graphics g)
    {
        Controller.paintMazeUpdate(g);
    }        
} 