/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.ui;


import com.oranda.pacdasher.uimodel.PacDasher;
import com.oranda.pacdasher.uimodel.UIModel;
import com.oranda.pacdasher.uimodel.util.UIModelConsts;
import com.oranda.pacdasher.uimodel.event.IPacListener;
import com.oranda.pacdasher.uimodel.event.PacEvent;
import com.oranda.pacdasher.uimodel.event.PacScoreEvent;
import com.oranda.pacdasher.uimodel.event.PacMoveEvent;
import com.oranda.util.Output;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.util.Random;

/**
 * Canvas showing score, lives, etc.
 */
public class InfoCanvas extends Canvas implements IPacListener
{

    private int score;
    private int highScore;
    
    protected Image backbuffer;
    protected Font font;
    protected int fontWidth;
    protected int fontHeight;
    
    protected Graphics offgc;
        
    public InfoCanvas()
    {
        setBackground(Color.BLACK);
    }
    
    // main use is initialization
    public void initialize()
    {
        this.backbuffer = createImage(getWidth(), getHeight());
        this.offgc = backbuffer.getGraphics();        
        this.fontHeight = UIModelConsts.FONT_SIZE_SCORE;
        // Monospaced maps to Courier                
        this.font = new Font("Monospaced", Font.BOLD, fontHeight);
        FontMetrics fontMetrics = offgc.getFontMetrics(font);
        int [] widths = fontMetrics.getWidths();
        this.fontWidth = widths[0];
        setFont(font);
        setBackground(Color.BLACK);
        setForeground(Color.LIGHT_GRAY);
    }
    
    /* 
     * Application-triggered
     */        
    public void update(Graphics g)
    {    
        paint(g);        
    }

    /* 
     * System-triggered and may be called from update
     */    
    public void paint(Graphics g)
    {
        // create the offscreen buffer and associated Graphics
        if (backbuffer == null)
        {
            initialize();
        }
        
        offgc.setColor(Color.BLACK);
        offgc.fillRect(0, 0, getWidth(), getHeight());
        offgc.setColor(Color.LIGHT_GRAY);

 
        
        offgc.setFont(font);      
        
        int headingX = UIModelConsts.X_TILE_SIZE * 5;
        offgc.drawString("1UP", headingX, fontHeight - 5);
        int numDigits = (Integer.toString(score)).length();
        int scoreX = headingX + fontWidth * 4 - fontWidth * numDigits;
        offgc.drawString("" + this.score, scoreX, fontHeight * 2 - 10);
        
        
        headingX = UIModelConsts.X_TILE_SIZE * 11;
                
        offgc.drawString("HIGH SCORE", headingX, fontHeight - 5); 
        numDigits = (Integer.toString(this.highScore)).length();
        int highScoreX = headingX + fontWidth * 8 - fontWidth * numDigits;
        offgc.drawString("" + this.highScore, highScoreX, fontHeight * 2 - 10);  
        g.drawImage(backbuffer, 0, 0, this);
        //offgc.dispose();        
    }
    
    public void pacScoreChanged(PacScoreEvent pse)
    {
        this.score = pse.getScore();
        this.highScore = pse.getHighScore();
        //Output.print(Output.DEBUG, "pacScoreChanged " + score);
        repaint();
    }
        
    public void pacDasherMoved(PacMoveEvent pme) {}
    public void pacDasherGainedPower() {}
    public void pacDasherLivesChanged(PacEvent pe) {}
}