/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.ui;


import com.oranda.pacdasher.uimodel.PacDasher;
import com.oranda.pacdasher.uimodel.UIModel;
import com.oranda.pacdasher.uimodel.util.UIModelConsts;
import com.oranda.pacdasher.uimodel.event.IPacListener;
import com.oranda.pacdasher.uimodel.event.PacEvent;
import com.oranda.pacdasher.uimodel.event.PacScoreEvent;
import com.oranda.pacdasher.uimodel.event.PacMoveEvent;
import com.oranda.util.Output;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;


/**
 * Canvas showing score, lives, etc.
 */
public class LivesCanvas extends Canvas implements IPacListener
{

    // after this, this variable is depended of pacDasher.getNumLives()
    private int numLives = UIModelConsts.NUM_LIVES_INIT;

    protected Image backbuffer;
    protected Image iPacleft;
    protected Graphics offgc;
         
    public LivesCanvas()
    {        
        //this.pacDasher = UIModel.getInstance().getPacDasher();
        setFont(new Font("Helvetica", Font.BOLD, 16));
        setBackground(Color.BLACK);
        try
        {
            String pacLeftFilename = UIModelConsts.PATH_IMAGES + "pacLeft.gif";
            File fPacleft = new File(pacLeftFilename);
            iPacleft = (BufferedImage) ImageIO.read(fPacleft);
        } 
        catch (IOException ioe)
        {
            Output.print(Output.DEBUG, 
                "Could not load pacdasher image for lives canvas", ioe);
        }
    }
    
    public void initialize()
    {
        backbuffer = createImage(getWidth(), getHeight());
        offgc = backbuffer.getGraphics();
        offgc.setColor(Color.BLACK);
        offgc.fillRect(0, 0, getWidth(), getHeight());
    }
    
    /* 
     * Application-triggered
     */        
    public void update(Graphics g)
    {    
        paint(g);        
    }

    /* 
     * System-triggered and may be called from update
     */    
    public void paint(Graphics g)
    {
        // create the offscreen buffer and associated Graphics
        if (backbuffer == null)
        {
            initialize();
        }
        
        offgc.setColor(Color.BLACK);
        offgc.fillRect(0, 0, getWidth(), getHeight());
        
        int imageWidth = (UIModelConsts.X_PAC_DASHER_SIZE*2)/3;
        int imageHeight = (UIModelConsts.Y_PAC_DASHER_SIZE*2)/3;
        
        for (int i=0; i<this.numLives-1; i++)
        {
            int x = (i + 2) * (imageWidth * 4)/3;
            offgc.drawImage(iPacleft, x, 0, imageWidth, imageHeight, null);  
        }
        g.drawImage(backbuffer, 0, 0, this);
        //offgc.dispose();        
    }
    
    public void pacScoreChanged(PacScoreEvent pse)
    {
    }
    
    public void pacDasherLivesChanged(PacEvent pe)
    {
        
        numLives = ((PacDasher) pe.getSource()).getNumLives();
        repaint();
    }
        
    public void pacDasherMoved(PacMoveEvent pme) {}
    public void pacDasherGainedPower() {}
}