/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.ui;

import com.oranda.pacdasher.State;
import com.oranda.pacdasher.uimodel.AnimatedVisualObject;
import com.oranda.pacdasher.uimodel.Maze;
import com.oranda.pacdasher.uimodel.PacDasher;
import com.oranda.pacdasher.uimodel.UIModel;
import com.oranda.pacdasher.uimodel.VisualObject;
import com.oranda.pacdasher.uimodel.util.XY;
import com.oranda.pacdasher.uimodel.util.XYCoarse;
import com.oranda.pacdasher.uimodel.util.XYCManager;
import com.oranda.pacdasher.uimodel.util.UIModelConsts;

import com.oranda.util.Output;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.Collections;


/**
 * MazeRenderer - draws the current maze.
 * Effectively combines input from UIModel and State to draw
 * to the GUI.
 */ 
public class MazeRenderer
{
    //protected Image background;
    
    Font font;    
    
    public MazeRenderer(Canvas c)
    {
        int fontHeight = UIModelConsts.FONT_SIZE_SCORE + 4;
        // Monospaced maps to Courier
        this.font = new Font("Monospaced", Font.BOLD, fontHeight);
    }


    public void paintMazeUpdate(Graphics g, Maze maze)
    {
        if (g == null || maze == null)
        {
            throw new IllegalArgumentException();
        }
        //Output.print(Output.DEBUG, "************** UPDATE MAZE *******");

        int state = State.getState();
        if (state == State.NORMAL_STATE)
        { 
        	//long beginTime = System.currentTimeMillis();
            
            maze.renderDirtyAreas(g);
            renderAnimated(g);
            
            //long endTime = System.currentTimeMillis();
            //long delay = endTime - beginTime;
            //if (delay > 10)
            //{
            //	Output.print(Output.DEBUG, "delay in offscreen update: " + delay);
            //}
            //java.awt.Toolkit.getDefaultToolkit().sync();	
        }
    }
    
    public void paintMaze(Graphics g, Maze maze)
    {    
        if (g == null || maze == null)
        {
            throw new IllegalArgumentException();
        }
        //Output.print(Output.DEBUG, "************** PAINT MAZE *******");
                
        g.setColor(Color.BLACK);
        g.fillRect(0, 0, maze.getWidth(), maze.getHeight());
        maze.renderObjects(g);
        int state = State.getState();
        if (state == State.NORMAL_STATE || state == State.CAPTURE_STATE)
        {
            renderAnimated(g);
        }
        else if (state == State.READY_STATE)
        {
            Output.print(Output.DEBUG, "Calling renderReady()");
            renderReady(g);    
        }
        else if (state == State.GAME_READY_STATE)
        {
            Output.print(Output.DEBUG, "Calling renderGamePrompt()");
            renderGamePrompt(g);
        }
        else if (state == State.GAME_OVER_STATE)
        {
            Output.print(Output.DEBUG, "Calling renderGameOver()");
            renderGameOver(g);
        }
    }
    
    private int getState()
    {
        return State.getState();
    }
    
    private void renderAnimated(Graphics g)
    {
        List animatedList = UIModel.getInstance().getMaze().getAnimatedList();
        
        for (int i=0; i<animatedList.size(); i++)
        {
            AnimatedVisualObject avo 
                = (AnimatedVisualObject) animatedList.get(i);
            avo.render(g);
        }
    }

    /* 
     * Draws "READY!"
     */    
    private void renderReady(Graphics g)
    {
        Color saveColor = g.getColor();
        Font saveFont = g.getFont();
        g.setFont(font);  
        XYCManager xycm = XYCManager.getInstance();
        int xTileSize = UIModelConsts.X_TILE_SIZE;
        int yTileSize = UIModelConsts.Y_TILE_SIZE;
                
        g.setColor(Color.YELLOW);
        int xPos = 12 * xTileSize;
        int yPos = (37 * yTileSize)/2;
        g.drawString("READY!", xPos, yPos);
        XYCoarse xycTopLeft = xycm.createXYC(xPos/xTileSize, yPos/yTileSize - 2);
        XYCoarse xycBottomRight = xycm.createXYC(xPos/xTileSize + 6, yPos/yTileSize);
        
        g.setColor(saveColor);
        g.setFont(saveFont);        
    }

    
    /* 
     * Draws "PLAYER ONE" and "READY!"
     */
    private void renderGamePrompt(Graphics g)
    {
        Color saveColor = g.getColor();
        Font saveFont = g.getFont();
        g.setFont(font);  
        XYCManager xycm = XYCManager.getInstance();
        
        int xTileSize = UIModelConsts.X_TILE_SIZE;
        int yTileSize = UIModelConsts.Y_TILE_SIZE;
        
        g.setColor(Color.CYAN);
        int xPos = 10 * xTileSize;
        int yPos = (25 * yTileSize)/2;
        g.drawString("PLAYER ONE", xPos, yPos);
        XYCoarse xycTopLeft = xycm.createXYC(xPos/xTileSize, yPos/yTileSize - 2);
        XYCoarse xycBottomRight 
            = xycm.createXYC(xPos/xTileSize + 10,  yPos/yTileSize);
        renderReady(g);
        
        g.setColor(saveColor);
        g.setFont(saveFont);
    }

    /* 
     * Draws "GAME OVER"
     */    
    private void renderGameOver(Graphics g)
    {
        Color saveColor = g.getColor();
        Font saveFont = g.getFont();
        g.setFont(font);  
        XYCManager xycm = XYCManager.getInstance();
        int xTileSize = UIModelConsts.X_TILE_SIZE;
        int yTileSize = UIModelConsts.Y_TILE_SIZE;
                
        g.setColor(Color.RED);
        int xPos = (21 * xTileSize)/2;
        int yPos = (37 * yTileSize)/2;
        g.drawString("GAME OVER", xPos, yPos);
        XYCoarse xycTopLeft = xycm.createXYC(xPos/xTileSize, yPos/yTileSize - 2);
        XYCoarse xycBottomRight = xycm.createXYC(xPos/xTileSize + 10, yPos/yTileSize);
        
        g.setColor(saveColor);
        g.setFont(saveFont);        
    }        
        
    public static void main(String[] args)
    {
    /*    final Maze maze = MazeFactory.create();
        final MazeRenderer renderer = new MazeRenderer(maze);
        
        GraphicsEnvironment graphicsEnvironment
            = GraphicsEnvironment.getLocalGraphicsEnvironment();
        GraphicsDevice graphicsDevice 
            = graphicsEnvironment.getDefaultScreenDevice();
        GraphicsConfiguration graphicsConfiguration
            = graphicsDevice.getDefaultConfiguration();
    
        Frame f = new Frame(graphicsConfiguration);
        int w=480, h=480;
        f.setSize(w, h);
        f.setVisible(true);
        Canvas c = new Canvas() {
          
            public void paint (Graphics g)
            {
                
                renderer.drawMaze(g);
            }
        };
        c.setSize(w, h);
        c.repaint();
        f.add(c);
        */
    }
    
}    