/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.ui;

import com.oranda.pacdasher.uimodel.util.DirectionCode;
import com.oranda.pacdasher.uimodel.MazeFactory;
import com.oranda.pacdasher.uimodel.Maze;
import com.oranda.pacdasher.uimodel.Ghost;
import com.oranda.pacdasher.uimodel.PosAndDirection;
import com.oranda.pacdasher.uimodel.PacDasher;
import com.oranda.pacdasher.uimodel.UIModel;
import com.oranda.pacdasher.uimodel.util.UIModelConsts;
import com.oranda.pacdasher.uimodel.util.XY;
import com.oranda.util.Output;

import java.awt.Canvas;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.geom.GeneralPath;
import java.awt.Polygon;
import java.awt.image.BufferStrategy;

import java.util.List;
import java.util.Iterator;

import javax.imageio.ImageIO;

/**
 * Main canvas for PacDasher
 */
public class PacCanvas extends Canvas
{
    protected Image backbuffer;
    
    protected PacDasher pacDasher;
     protected PosAndDirection PosAndDirection;
    
    protected MazeRenderer mazeRenderer;
    //protected Graphics offgc;
    
    protected Polygon generalPath;

    protected static int count = 0;
        
    public PacCanvas()
    {
        this.pacDasher = UIModel.getInstance().getPacDasher();
        this.PosAndDirection = pacDasher.getPosAndDirection();
                
        setBackground(UIModelConsts.MAZE_BG_COLOR);
        
        repaint();
        //super();
    }
    
    
    // main use is initialization
    public void initialize(Graphics g)
    {
        backbuffer = createImage(getWidth(), getHeight());
        Graphics offgc = backbuffer.getGraphics();
        //offgc.translate(UIModelConsts.FRAME_EDGE_SIZE, 
        //        UIModelConsts.FRAME_BAR_SIZE);
        Output.print(Output.DEBUG, "finished initialization");            
         //generalPath = new Polygon(); 

    }
    

    /* 
     * Application-triggered - for incremental drawing
     */    
    public void update(Graphics g)
    {
        
        //Output.print(Output.DEBUG, "update");
        
        if (backbuffer == null)
        {
            initialize(g);
        }
        
        
        Graphics offgc = backbuffer.getGraphics();
        GUI.paintMazeUpdate(offgc);               
        
        int yTileSize = UIModelConsts.Y_TILE_SIZE;
    
        //long beforeDraw = System.currentTimeMillis(); 
        //g.setClip(100,100,300,300);
        g.drawImage(backbuffer, 0, 0, this);
        //long afterDraw = System.currentTimeMillis();
        //long delay = afterDraw - beforeDraw;
        //if (delay > 10)
    //    {
        //    Output.print(Output.DEBUG, "PacCanvas drawing delay was " + delay);
        //}
    }



    
    /* 
     * System-triggered - if damaged (was obscured), redraw maze
     */
    
    public void paint(Graphics g)
    {
        //Output.print(Output.DEBUG, "paint");

        // create the offscreen buffer and associated Graphics
        Graphics offgc = null;
        if (backbuffer == null)
        {
            initialize(g);
            offgc = backbuffer.getGraphics();
            GUI.paintMaze(offgc);
        }        

        g.drawImage(backbuffer, 0, 0, this);
        
        if (offgc != null)
        {
            offgc.dispose();
        }

    }
}
