/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;

import com.oranda.pacdasher.uimodel.util.*;

import java.awt.Color;
import java.awt.Graphics;
import java.util.List;

public class BoxWall extends Wall
{    
    public BoxWall(Color color)
    {
        super(color);
    }
                
    public void render(Graphics g, XYCoarse xyCoarse)
    {
        g.setColor(color);
        g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + 4,
                getXTopLeft(xyCoarse, 0) + UIModelConsts.X_TILE_SIZE,
                getYTopLeft(xyCoarse, 0) + 4);
        g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + 8,
                getXTopLeft(xyCoarse, 0) + UIModelConsts.Y_TILE_SIZE,
                getYTopLeft(xyCoarse, 0) + 8);
    }
    
    public void render(Graphics g, XYCoarse xyCoarse, List neighbors)
    {
        g.setColor(color);
        //Output.print(Output.DEBUG, "xyCoarse " + xyCoarse + ", neighbors" + neighbors);

        if (neighbors.contains(DirectionCode.UP) 
            && neighbors.contains(DirectionCode.RIGHT))
        {
            
            g.drawLine(getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 4,
                getXTopLeft(xyCoarse, 0) + 16,
                getYTopLeft(xyCoarse, 0) + 4);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + 8,
                getXTopLeft(xyCoarse, 0) + 16,
                getYTopLeft(xyCoarse, 0) + 8);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + 8);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 4);

        }
        else if (neighbors.contains(DirectionCode.DOWN) 
            && neighbors.contains(DirectionCode.RIGHT))
        {
            g.drawLine(getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 8,
                getXTopLeft(xyCoarse, 0) + 16,
                getYTopLeft(xyCoarse, 0) + 8);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + 4,
                getXTopLeft(xyCoarse, 0) + 16,
                getYTopLeft(xyCoarse, 0) + 4);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + 4,
                getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + 16);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 8,
                getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 16);
                
        }
        else if (neighbors.contains(DirectionCode.LEFT) 
            && neighbors.contains(DirectionCode.DOWN))
        {
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + 8,
                getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + 8);
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + 4,
                getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 4);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 4,
                getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 16);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + 16,
                getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + 8);
                
        }
        else if (neighbors.contains(DirectionCode.UP) 
            && neighbors.contains(DirectionCode.LEFT))
        {
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + 4,
                getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + 4);
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + 8,
                getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 8);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 0,
                getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + 8);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + 4);        
        }
        else if (neighbors.contains(DirectionCode.UP) 
            && neighbors.contains(DirectionCode.DOWN))
        {
            g.drawLine(getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + 4,
                getYTopLeft(xyCoarse, 0) + UIModelConsts.X_TILE_SIZE);
            g.drawLine(getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + 8,
                getYTopLeft(xyCoarse, 0) + UIModelConsts.X_TILE_SIZE);
        }
        else
        {
            render(g, xyCoarse);
        }
    
    }    
    
    public static int getDiameter()
    {
        return diameter;
    }

}