/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;

import com.oranda.pacdasher.uimodel.util.*;

import java.util.Random;

class FlightMoveAttempt extends MoveAttempt
{
    protected static Random randomGenerator = new Random();
    
    private static MoveAttempt moveAttempt;
    private FlightMoveAttempt() { }
    public static MoveAttempt getInstance() 
    { 
        if (moveAttempt == null)
        {
            moveAttempt = new FlightMoveAttempt();
        }
        return moveAttempt; 
    }
            
    public DirectionCode propose(Ghost ghost, XY pacDasherXy, 
        java.util.List allowedDirections)
    {
        DirectionCode desiredDirectionCode;
        XY xy = ghost.getXY();
 
        int up = xy.getY() - pacDasherXy.getY();
        int left = xy.getX() - pacDasherXy.getX(); 
        int down = pacDasherXy.getY() - xy.getY();
        int right = pacDasherXy.getX() - xy.getX();
        
        
        DirectionCode curDirectionCode = ghost.getPosAndDirection().getCurDirectionCode();
        
        if (allowedDirections.contains(DirectionCode.UP)
            && up <= left && up <= down && up < right 
            && !curDirectionCode.equals(DirectionCode.DOWN))
        {
            desiredDirectionCode = DirectionCode.UP;
        }
        else if (allowedDirections.contains(DirectionCode.LEFT)
            && left <= down && left <= right 
            && !curDirectionCode.equals(DirectionCode.RIGHT))
        {
            desiredDirectionCode = DirectionCode.LEFT;
        }
        else if (allowedDirections.contains(DirectionCode.DOWN)
            && down <= right 
            && !curDirectionCode.equals(DirectionCode.UP))
        {
            desiredDirectionCode = DirectionCode.DOWN;
        }
        else if (allowedDirections.contains(DirectionCode.RIGHT) 
            && !curDirectionCode.equals(DirectionCode.LEFT))
        {
            desiredDirectionCode = DirectionCode.RIGHT;
        }
        else
        {
            int r = randomGenerator.nextInt(allowedDirections.size());
            desiredDirectionCode = (DirectionCode) allowedDirections.get(r);
        }
                   
        return desiredDirectionCode;
                                 
    }
}    