/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;



import com.oranda.pacdasher.uimodel.event.IPacListener;
import com.oranda.pacdasher.uimodel.event.PacEvent;
import com.oranda.pacdasher.uimodel.event.PacScoreEvent;
import com.oranda.pacdasher.uimodel.event.PacMoveEvent;
import com.oranda.pacdasher.uimodel.util.*;
import com.oranda.util.Output;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;

public  class Ghost extends MobileVisualObject implements IPacListener
{
    protected static final int TIME_FLIGHT_NORMAL = 400;
    protected static final int TIME_FLIGHT_TOTAL    = 550;
    protected static final int TIME_SCATTER = 300;
    protected static final int PERIOD_SCATTER = 1500;
    
    protected static final String imagesPath = UIModelConsts.PATH_IMAGES;
    protected static final String gFlightFilename = "gFlight.gif";
    protected static final String gFlightWhiteFilename = "gFlightWhite.gif";

    protected XYCoarse xyCoarse;
    protected Image curImg, iRight, iLeft, iUp, iDown, iFlight, iFlightWhite;
    protected XY pacDasherXy;
    protected DirectionCode pacDasherDirectionCode;
    protected GhostStrategy ghostStrategy;

    protected static final int scoreValue = 200;    

    
    public Ghost(XYCoarse xyCoarseInit)
    {
        super(xyCoarseInit);
        
        // default until set
        this.pacDasherXy = new XY(0, 0); 
        this.pacDasherDirectionCode = DirectionCode.LEFT;
        // default
        this.ghostStrategy = new GhostStrategyBlinky(this);
    }
    
    /*
    public void init()
    {        
        ghostStrategy.setState(GhostStrategy.NORMAL_STATE);
    }*/
    
    public void loadFiles(String iRightFilename, String iLeftFilename, 
        String iUpFilename, String iDownFilename)
    {
        try
        {
            File fRight = new File(imagesPath + iRightFilename);
            iRight = (BufferedImage) ImageIO.read(fRight);
            File fLeft = new File(imagesPath + iLeftFilename);
            iLeft = (BufferedImage) ImageIO.read(fLeft);
            File fUp = new File(imagesPath + iUpFilename);
            iUp = (BufferedImage) ImageIO.read(fUp);
            File fDown = new File(imagesPath + iDownFilename);
            iDown = (BufferedImage) ImageIO.read(fDown);
            File fFlight = new File(imagesPath + gFlightFilename);
            iFlight = (BufferedImage) ImageIO.read(fFlight);
            File fFlightWhite = new File(imagesPath + gFlightWhiteFilename);
            iFlightWhite = (BufferedImage) ImageIO.read(fFlightWhite);
            curImg = iLeft; 
            
        } 
        catch (IOException ioe)
        {
            Output.print(Output.DEBUG, "Error loading image files for ghost", ioe);
        }        
    }
    
    public int getState()
    {
        return ghostStrategy.getState();
    }
    
    public Image getAppropriateImage(DirectionCode directionCode)
    {
        int virtualTimeSinceFlight = 
            ghostStrategy.getVirtualTimeSinceFlight();
            
        if (ghostStrategy.getState() == GhostStrategy.FLIGHT_STATE
            && virtualTimeSinceFlight > TIME_FLIGHT_NORMAL
            && virtualTimeSinceFlight%16 > 8  )
        {
            return iFlightWhite;
        }
        else if (ghostStrategy.getState() == GhostStrategy.FLIGHT_STATE)
        {
            return iFlight;
        }
        else if (directionCode == DirectionCode.DOWN)
        {
            return iDown;
        } 
        else if (directionCode == DirectionCode.UP)
        {
            return iUp;
        } 
        else if (directionCode == DirectionCode.LEFT)
        {
            return iLeft;
        } 
        else if (directionCode == DirectionCode.RIGHT)
        {
            return iRight;
        } 
        else
        {
            return null;
        }                
    }
    
    public PosAndDirection getPosAndDirection()
    {
        return this.posAndDirection;
    }
    
    public void setPosAndDirection(PosAndDirection posAndDirection)
    {
        this.posAndDirection = posAndDirection;
        //init();
    }
    
    public void reset()
    {
        super.reset();
        ghostStrategy.setState(GhostStrategy.NORMAL_STATE);
        //Output.print(Output.DEBUG, "posAndDirection after " + posAndDirection);
        //ghostStrategy.setPosAndDirection(posAndDirection);    
    }
        
    public void render(Graphics g)
    {
        int xTopCorner = posAndDirection.getX() - UIModelConsts.X_TILE_SIZE;
        int yTopCorner = posAndDirection.getY() - UIModelConsts.Y_TILE_SIZE;
        //clip(g, xTopCorner, yTopCorner);
        
        Image img = getAppropriateImage(posAndDirection.getCurDirectionCode());
        if (img != null)
        {
            curImg = img;
        }        
        
        g.drawImage(curImg, xTopCorner, yTopCorner,
            UIModelConsts.GHOST_DIAMETER, 
            UIModelConsts.GHOST_DIAMETER, null);
    }


    

    public void move()
    {
        ghostStrategy.move(this.posAndDirection, 
            this.pacDasherXy, 
            this.pacDasherDirectionCode);
    }
    
    public void clip(Graphics g, int x, int y)
    {
        g.setClip(null);
        g.clipRect(x - 5, y - 5, UIModelConsts.GHOST_DIAMETER + 10, 
            UIModelConsts.GHOST_DIAMETER + 10);
    }
    
    // callbacks
    public void pacScoreChanged(PacScoreEvent pse) {}       
       public void pacDasherMoved(PacMoveEvent pme) 
       {
           XY xy = pme.getXy();
           this.pacDasherXy = xy;    
           this.pacDasherDirectionCode = pme.getDirectionCode(); 
       }
       
       public void pacDasherGainedPower() 
       {
           ghostStrategy.setState(GhostStrategy.FLIGHT_STATE);
       }
       
       public void pacDasherLivesChanged(PacEvent pe)
       {
           setPosAndDirection(initPosAndDirection);
       }

    
    public int getScoreValue()
    {
        return scoreValue;
    }
}