/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;

import com.oranda.pacdasher.uimodel.util.*;

/*
 * Inky likes to try the directions: Right, Left, Down, Up in order
 * except when he is near to PacDasher - then he is like Blinky
 */
public class GhostStrategyInky extends GhostStrategy
{

    public GhostStrategyInky(Ghost ghost)
    {
        super(ghost);
    }



    public void tryMove(PosAndDirection posAndDirection,
        XY pacDasherXy, DirectionCode pacDasherDirectionCode,
        MoveAttempt moveAttempt)
    {
        XY xy = ghost.getXY();
        int verticalDistance = Math.abs(xy.getY() - pacDasherXy.getY());
        int horizontalDistance = Math.abs(xy.getX() - pacDasherXy.getX());
        
        int mazeWidth = UIModel.getInstance().getMaze().getWidth();
        int sumDistance = verticalDistance + horizontalDistance;
        int nearLength = UIModelConsts.Y_TILE_SIZE * 16;
        if (sumDistance < nearLength)
        {  
            super.tryMove(posAndDirection, pacDasherXy, 
                pacDasherDirectionCode, moveAttempt);    
        }
        else
        {    
            if (state == NORMAL_STATE)
            {
                moveAttempt = new NormalMoveAttemptInky();
            }
            tryMoveDirections(posAndDirection,
                (java.util.List) possibleDirections.clone(),
                pacDasherXy, moveAttempt);                    
        }
     }
     


    /*
     * Default algorithm for Blinky supposed to be
     * 1. Always try to close the horizontal or vertical distance
     *    between himself or PacDasher, whichever is shorter.
     * (2. If distances equal, use priority order UP, LEFT, DOWN, RIGHT)
     */    
    class NormalMoveAttemptInky extends MoveAttempt
    {
        public DirectionCode propose(Ghost ghost, XY pacDasherXy, 
            java.util.List allowedDirections)
        {
            DirectionCode desiredDirectionCode;
            XY xy = ghost.getXY();
              
            DirectionCode curDirectionCode = ghost.getPosAndDirection().getCurDirectionCode();

            
            if (allowedDirections.contains(DirectionCode.UP)
                && !curDirectionCode.equals(DirectionCode.DOWN))
            {
                desiredDirectionCode = DirectionCode.UP;
            }
            else if (allowedDirections.contains(DirectionCode.LEFT)
                && !curDirectionCode.equals(DirectionCode.RIGHT))
            {
                desiredDirectionCode = DirectionCode.LEFT;
            }
            else if (allowedDirections.contains(DirectionCode.DOWN)
                && !curDirectionCode.equals(DirectionCode.UP))
            {
                desiredDirectionCode = DirectionCode.DOWN;
            }
            else if (allowedDirections.contains(DirectionCode.RIGHT) 
                && !curDirectionCode.equals(DirectionCode.LEFT))
            {
                desiredDirectionCode = DirectionCode.RIGHT;
            }
            else
            {
                int r = randomGenerator.nextInt(allowedDirections.size());
                desiredDirectionCode = (DirectionCode) allowedDirections.get(r);
            }
                       
            return desiredDirectionCode;
                                        
        }
    }
     
     
}