/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;

import com.oranda.pacdasher.uimodel.util.*;

public class GhostStrategyPinky extends GhostStrategy
{

    public GhostStrategyPinky(Ghost ghost)
    {
        super(ghost);
    }

    /*
     * Pinky is like Blinky at a distance
     * Otherwise she tries to move in front of PacDasher
     */
    public void tryMove(PosAndDirection posAndDirection,
        XY pacDasherXy, DirectionCode pacDasherDirectionCode,
        MoveAttempt moveAttempt)
    {
        XY xy = ghost.getXY();
        int verticalDistance = Math.abs(xy.getY() - pacDasherXy.getY());
        int horizontalDistance = Math.abs(xy.getX() - pacDasherXy.getX());
        
        int mazeWidth = UIModel.getInstance().getMaze().getWidth();
        int sumDistance = verticalDistance + horizontalDistance;
        int nearLength = UIModelConsts.Y_TILE_SIZE * 6;
        if (sumDistance > mazeWidth || sumDistance < nearLength ||
            !((xy.getX()%UIModelConsts.X_TILE_SIZE == 0 && ghost.isMovingHorizontally())
                || (xy.getY()%UIModelConsts.Y_TILE_SIZE == 0 && ghost.isMovingVertically())))
        {  
            super.tryMove(posAndDirection, pacDasherXy, null, moveAttempt);    
        }
        else
        {
            // try to move in front of PacDasher
            int xPacPredicted = pacDasherXy.getX();
            int yPacPredicted = pacDasherXy.getY();
            if (pacDasherDirectionCode.equals(DirectionCode.UP))
            {
                yPacPredicted -= nearLength;
            }
            else if (pacDasherDirectionCode.equals(DirectionCode.DOWN))
            {
                yPacPredicted += nearLength;
            }
            else if (pacDasherDirectionCode.equals(DirectionCode.LEFT))
            {
                xPacPredicted -= nearLength;
            }
            else if (pacDasherDirectionCode.equals(DirectionCode.RIGHT))
            {
                xPacPredicted += nearLength;
            }
            
            XY xyPacPredicted = new XY(xPacPredicted, yPacPredicted);
             
            tryMoveDirections(posAndDirection, (java.util.List) possibleDirections.clone(),
                xyPacPredicted, moveAttempt);
                        
        }
     }
}