/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;


import com.oranda.pacdasher.State;
import com.oranda.pacdasher.ui.DirtyArea;
import com.oranda.pacdasher.uimodel.util.*;

import com.oranda.util.Output;

import java.awt.Graphics;
import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import java.util.HashSet;
import java.util.Set;

/**
 * Maze - main class under UIModel
 */
public class Maze 
{    
    
    // pixel dimensions 
    private final int width;
    private final int height;
    
    private final GhostBlinky ghostBlinky;
    private final GhostPinky ghostPinky;
    private final GhostInky ghostInky;
    private final GhostClyde ghostClyde;
    
    private final ArrayList ghostList;
    private final ArrayList mobileList;
    private final ArrayList animatedList;
    
    private Color wallColor;
    
    
    
    /*
     * staticBackground = XYCoarse -> walls and background tiles
     *                      (placed on blocks of grid)
     * dynamicBackground = XYCoarse -> dots, energizers, etc.
     *                      (placed on intersections of grid)
     */
    private MazeData staticBackground;
    private MazeData dynamicBackground;
    
    private int xCoarseBoundary;
    private int yCoarseBoundary;
    
    // local to getNeighbors but hate gc();
    private ArrayList neighbors = new ArrayList(2);
            
    private boolean isMazeFinished = false;
    
    private Dot sampleDot = new Dot();
    private Energizer sampleEnergizer = new Energizer(null);
    
    public Maze(Direction pacDirection, int[][] walls, int[][] boxWalls, 
        int[][] clearArea, int[][] clearAreaExtra,
        int[][] gates, int[][] energizers, int[][] pacDashers, 
        int[][] ghostsGeneric, 
        int[][] ghostsBlinky, int[][] ghostsPinky, int[][] ghostsInky, 
        int[][] ghostsClyde,  Color wallColor, Color gateColor) 
    {         
        isMazeFinished = false;
        
        xCoarseBoundary = UIModelConsts.MAZE_WIDTH;
        yCoarseBoundary = UIModelConsts.MAZE_HEIGHT;
        this.width = (this.xCoarseBoundary + 1) * UIModelConsts.X_TILE_SIZE;
        this.height = (this.yCoarseBoundary + 1) * UIModelConsts.Y_TILE_SIZE;


        /*
         * INITIALIZATION OF MOBILE OBJECTS
         */        
         
        this.ghostBlinky = new GhostBlinky(getXYCoarse(ghostsBlinky));
        this.ghostPinky = new GhostPinky(getXYCoarse(ghostsPinky));
        this.ghostInky = new GhostInky(getXYCoarse(ghostsInky));
        this.ghostClyde = new GhostClyde(getXYCoarse(ghostsClyde));

        XYCoarse pacDPos = new XYCoarse(pacDashers[0][0], pacDashers[0][1]);
        getPacDasher().initPos(pacDPos);
        
        this.ghostList = new ArrayList(4);
        this.ghostList.add(ghostBlinky);
        this.ghostList.add(ghostPinky);
        this.ghostList.add(ghostInky);
        this.ghostList.add(ghostClyde);
        
        this.mobileList = new ArrayList(5);
        this.mobileList.addAll(ghostList);

        this.mobileList.add(getPacDasher());

                
        /*
         * INITIALIZATION OF NON-MOBILE OBJECTS
         */        
        staticBackground = new MazeData(xCoarseBoundary, yCoarseBoundary + 1);
        dynamicBackground = new MazeData(xCoarseBoundary, yCoarseBoundary + 1);
        
        for (int xCoarse=0; xCoarse<=xCoarseBoundary; xCoarse++)
        {
            for (int yCoarse=0; yCoarse<=yCoarseBoundary; yCoarse++)
            {
                staticBackground.put(xCoarse, yCoarse, new Background());
                dynamicBackground.put(xCoarse, yCoarse, null);
            }
        }    
        
        Wall wall = new Wall(wallColor);
        setStaticBackground(walls, wall);
        BoxWall boxWall = new BoxWall(wallColor);
        setStaticBackground(boxWalls, boxWall);
        
        Gate gate = new Gate(gateColor);
        setDynamicBackground(gates, gate);
        Dot dot = new Dot();
        setDynamicBackgroundRemainder(dot, clearArea);                

        this.animatedList = new ArrayList(9);
        this.animatedList.addAll(this.mobileList);
                    
        setDynamicBackgroundEnergizer(energizers); // overwrite dots
        setDynamicBackground(clearAreaExtra, null); // overwrite dots

                
        this.wallColor = wallColor;
    }
    
    public boolean isMazeFinished()
    {
        return isMazeFinished;
    }
    
    protected void checkMazeFinished()
    {
        for (int xCoarse=0; xCoarse<=xCoarseBoundary; xCoarse++)
        {
            for (int yCoarse=0; yCoarse<=yCoarseBoundary; yCoarse++)
            {
                VisualObject vo = (VisualObject) dynamicBackground.get(xCoarse, yCoarse);
                if (vo != null && (vo instanceof Dot
                     || vo instanceof Energizer))
                {
                    isMazeFinished = false;
                    return;
                }
            }
        }        
        Output.print(Output.DEBUG, "maze finished!");
        isMazeFinished = true;
        UIModel.setState(State.BEFORE_LEVEL_STATE);
    }
        
    public void setStaticBackground(int[][] array, VisualObject visualObject)
    {    
        for (int i=0; i<array.length; i++)
        {
             int xCoarse = array[i][0];
             int yCoarse = array[i][1];
             this.staticBackground.put(xCoarse, yCoarse, visualObject);         
        }
    }
        
    public void setDynamicBackground(int[][] array, VisualObject visualObject)
    {    
        for (int i=0; i<array.length; i++)
        {
             int xCoarse = array[i][0];
             int yCoarse = array[i][1];
             this.dynamicBackground.put(xCoarse, yCoarse, visualObject);         
        }
    }
    
    public void setDynamicBackgroundEnergizer(int[][] array)
    {    
        for (int i=0; i<array.length; i++)
        {
             int xCoarse = array[i][0];
             int yCoarse = array[i][1];
             XYCoarse xyCoarse 
                 = XYCManager.getInstance().createXYC(xCoarse, yCoarse);
             Energizer energizer = new Energizer(xyCoarse);
             animatedList.add(energizer);
             this.dynamicBackground.put(xCoarse, yCoarse, energizer);         
        }
    }
    
    public void removeAnimated(XYCoarse xyCoarse, 
        AnimatedVisualObject avoToRemove)
    {
        for (int i=0; i<animatedList.size(); i++)
        {
            AnimatedVisualObject avo 
                = (AnimatedVisualObject) animatedList.get(i);
            if (avo.getClass().equals(avoToRemove.getClass()))
            {
                XYCoarse xyCoarseAnimated = avo.getXYCoarse();
                if (xyCoarse.equals(xyCoarseAnimated))
                {
                    animatedList.remove(avo);
                }
            }
        }
    }
    
    public void renderObjects(Graphics g)
    {
        // non-intersection based
        for (int xCoarse=0; xCoarse<=xCoarseBoundary; xCoarse++)
        {
            for (int yCoarse=0; yCoarse<=yCoarseBoundary; yCoarse++)
            {
                XYCoarse xyCoarse 
                     = XYCManager.getInstance().createXYC(xCoarse, yCoarse);
                VisualObject vo = (VisualObject) staticBackground.get(xCoarse, yCoarse);
                if (vo == null)
                {
                }
                else if (vo.isNeighborly())
                {
                    List neighbors = getNeighbors(xyCoarse, vo);
                    vo.render(g, xyCoarse, neighbors);
                }
                else
                {
                    vo.render(g, xyCoarse);
                }
            }
        }
        
        // intersection-based like Dot or Energizer
        for (int xCoarse=0; xCoarse<=xCoarseBoundary; xCoarse++)
        {
            for (int yCoarse=0; yCoarse<=yCoarseBoundary; yCoarse++)
            {
                XYCoarse xyCoarse 
                     = XYCManager.getInstance().createXYC(xCoarse, yCoarse);
                VisualObject vo = (VisualObject) dynamicBackground.get(xCoarse, yCoarse);
                if (vo != null)
                {
                    vo.render(g, xyCoarse);
                }
            }
        }        
    }

    /*
     * @return a List of DirectionCode's
     */
    public List getNeighbors(XYCoarse xyCoarse, VisualObject visualObject)
    {        
        neighbors.clear();
        int xCoarse = xyCoarse.getX();
        int yCoarse = xyCoarse.getY();
        
        if (visualObject.equals(staticBackground.get(xCoarse-1, yCoarse)))
        {
            neighbors.add(DirectionCode.LEFT);
        }
        if (visualObject.equals(staticBackground.get(xCoarse+1, yCoarse)))
        {
            neighbors.add(DirectionCode.RIGHT);
        }
        if (visualObject.equals(staticBackground.get(xCoarse, yCoarse-1)))
        {
            neighbors.add(DirectionCode.UP);
        }
        if (visualObject.equals(staticBackground.get(xCoarse, yCoarse+1)))
        {
            neighbors.add(DirectionCode.DOWN);
        }                
        return neighbors;
    }
      
     /*   
    public void renderExtraDirtyAreas(Graphics g, List extraDirtyAreas)
    {
        for (int i=0; i<extraDirtyAreas.size(); i++)
        {
            DirtyArea dirtyArea = (DirtyArea) extraDirtyAreas.get(i);
            renderDirtyArea(g, dirtyArea);
        }    
    }
    */
    
    public void renderDirtyAreas(Graphics g)
    {    
        for (int i=0; i<mobileList.size(); i++)
        {
            MobileVisualObject mvo 
                = (MobileVisualObject) this.mobileList.get(i);
            DirtyArea dirtyArea = mvo.getDirtyArea();
            renderDirtyArea(g, dirtyArea);
        }        
    }

    public void renderDirtyArea(Graphics g, DirtyArea dirtyArea)
    {
    	
        if (dirtyArea == null || dirtyArea.getXYCLeftTop() == null)
        {
            return;
        }

        int xycLeft = dirtyArea.getXYCLeftTop().getX();
        int xycTop = dirtyArea.getXYCLeftTop().getY();
        int xycRight = dirtyArea.getXYCRightBottom().getX();
        int xycBottom = dirtyArea.getXYCRightBottom().getY();
    
        XYCManager xycm = XYCManager.getInstance();
        
                
        // render static background
        for (int xCoarse = xycLeft; xCoarse <= xycRight; xCoarse++)
        {
            for (int yCoarse = xycTop; yCoarse <= xycBottom; yCoarse++)
            {            
                VisualObject vo = staticBackground.get(xCoarse, yCoarse);
                if (vo != null && vo instanceof Background)
                {
                    XYCoarse xyCoarse = xycm.createXYC(xCoarse, yCoarse);
                    vo.render(g, xyCoarse);
                }
            }
        }
                
        // render dynamic background - dots, etc.        
        for (int xCoarse = xycLeft; xCoarse <= xycRight  + 1; xCoarse++)
        {
            for (int yCoarse = xycTop; yCoarse <= xycBottom + 1; yCoarse++)
            {
            	
                VisualObject vo = dynamicBackground.get(xCoarse, yCoarse);
                if (vo != null)
                {
                    XYCoarse xyCoarse = xycm.createXYC(xCoarse, yCoarse);
                    vo.render(g, xyCoarse);
                }
                
            }
        }       
         
    }    
         

    /*
     * Fill blank parts of maze with specified visualObject (dots)
     */
    public void setDynamicBackgroundRemainder(VisualObject visualObject,
        int[][] clearArea)
    {
        for (int xCoarse=1; xCoarse<=xCoarseBoundary; xCoarse++)
        {
            for (int yCoarse=1; yCoarse<=yCoarseBoundary; yCoarse++)
            {
                if (xCoarse >= clearArea[0][0] && xCoarse <= clearArea[1][0]
                    && yCoarse >= clearArea[0][1] && yCoarse <= clearArea[1][1])
                {
                    continue;
                } 
                
                XYCoarse xyCoarse0 
                     = XYCManager.getInstance().createXYC(xCoarse, yCoarse);
                XYCoarse xyCoarse1 
                     = XYCManager.getInstance().createXYC(xCoarse - 1, yCoarse);
                XYCoarse xyCoarse2 
                     = XYCManager.getInstance().createXYC(xCoarse, yCoarse - 1);
                XYCoarse xyCoarse3 
                     = XYCManager.getInstance().createXYC(xCoarse - 1, yCoarse - 1);

                if (isFree(xyCoarse0) && isFree(xyCoarse1)
                    && isFree(xyCoarse2) && isFree(xyCoarse3))
                {
                    dynamicBackground.put(xCoarse, yCoarse, visualObject);
                }
            }
        }  
    }    
    
    private boolean isFree(XYCoarse xyCoarse)
    {
        VisualObject voStatic = (VisualObject) 
            staticBackground.get(xyCoarse.getX(), xyCoarse.getY());
        VisualObject voDynamic = (VisualObject) 
            dynamicBackground.get(xyCoarse.getX(), xyCoarse.getY());
        if (voStatic instanceof Background
            && (voDynamic == null || !(voDynamic instanceof Gate)))
        {
            return true;
        }
        return false;
    }
    
    public boolean isBlocked(MobileVisualObject mvo, int xCoarse, int yCoarse)
    {
        VisualObject voStatic = (VisualObject) staticBackground.get(xCoarse, yCoarse);
        
        if (voStatic != null)
        {
            if (voStatic instanceof Wall || voStatic instanceof BoxWall)
            {
                return true;
            }
        }
        VisualObject voDynamic = (VisualObject) dynamicBackground.get(xCoarse, yCoarse);
        if (voDynamic != null)
        {
            if (voDynamic instanceof Gate && mvo instanceof PacDasher)
            {
                return true;
            }
        }
        return false;
    }
    
    
    
    public XYCoarse getXYCoarse(int[][] arraySingleItem)
    {
        return XYCManager.getInstance().createXYC(
            arraySingleItem[0][0], arraySingleItem[0][1]);    
    }
    
    
    private void setPosAndDirection(Ghost ghost, int[][] arrayInt2D)
    {
        Direction directionInit = new Direction(DirectionCode.LEFT);
        XYCoarse xyCoarseInit 
           = XYCManager.getInstance().createXYC(arrayInt2D[0][0], arrayInt2D[0][1]);
        ghost.setPosAndDirection(
            new PosAndDirection(ghost, xyCoarseInit, directionInit));
    }    
    
    
    public void move()
    {    
    	
        for (int i=0; i<this.mobileList.size(); i++)
        {
            MobileVisualObject mvo 
                = (MobileVisualObject) this.mobileList.get(i);
            mvo.moveAndSetDirtyArea();
        }
        adjustModel();        
    }    

    public void resetMobiles()
    {
        for (int i=0; i<this.mobileList.size(); i++)
        {
            MobileVisualObject mvo 
                = (MobileVisualObject) this.mobileList.get(i);
            mvo.reset();
        }
    }
    
    
    public void adjustModel()
    {
        /* 
         * Check to see if PacDasher has eaten anything
         * we can restrict search to the dirty area
         */
         
        DirtyArea dirtyArea = getPacDasher().getDirtyArea();

        if (dirtyArea == null || dirtyArea.getXYCLeftTop() == null)
        {
            return;
        }
        
        int xPacLeft = dirtyArea.getXYCLeftTop().getX();
        int yPacTop = dirtyArea.getXYCLeftTop().getY();
        int xPacRight = dirtyArea.getXYCRightBottom().getX();
        int yPacBottom = dirtyArea.getXYCRightBottom().getY();

        int xLeft, yTop, xRight, yBottom, diameter;
                
        for (int xCoarse=xPacLeft; xCoarse<=xPacRight; xCoarse++)
        {
            for (int yCoarse=yPacTop; yCoarse<=yPacBottom; yCoarse++)
            {    
                XYCManager xycm = XYCManager.getInstance();
                XYCoarse xyCoarse = xycm.createXYC(xCoarse, yCoarse);
                VisualObject vo = (VisualObject) dynamicBackground.get(xCoarse, yCoarse);    
                if (vo != null && vo instanceof Dot)
                {            
                    diameter = Dot.getDiameter();
                    xLeft = Dot.getXTopLeft(xyCoarse, diameter); 
                    yTop = Dot.getYTopLeft(xyCoarse, diameter); 
                    xRight = xLeft + diameter;
                    yBottom = yTop + diameter;
                    if (getPacDasher().envelopes(xLeft, xRight, yTop, yBottom))
                    {
                        eatObject(sampleDot, xCoarse, yCoarse);

                    }
                }
                else if (vo != null && vo instanceof Energizer)
                {        
                    diameter = Energizer.getDiameter();
                    xLeft = Energizer.getXTopLeft(xyCoarse, diameter); 
                    yTop = Energizer.getYTopLeft(xyCoarse, diameter); 
                    xRight = xLeft + diameter;
                    yBottom = yTop + diameter;
                    if (getPacDasher().envelopes(xLeft, xRight, yTop, yBottom))
                    {
                        eatObject(sampleEnergizer, xCoarse, yCoarse);
                        removeAnimated(xyCoarse, sampleEnergizer);                        
                        getPacDasher().gainedPower();
                    }
                }
            }
        }  
  
        // has PacDasher run into a Ghost
        checkMobileCollision();    
    }
    
    protected void eatObject(VisualObject vo, int xCoarse, int yCoarse)
    {
        getPacDasher().addToScore(vo.getScoreValue());
        dynamicBackground.put(xCoarse, yCoarse, null);
        checkMazeFinished();
    }                        
        
    private PacDasher getPacDasher()
    {
        return UIModel.getInstance().getPacDasher();
    }
    
    private void checkMobileCollision()
    {
        int xPacCoreRadius = UIModelConsts.X_PAC_DASHER_SIZE/4;
        int yPacCoreRadius = UIModelConsts.Y_PAC_DASHER_SIZE/4;
        int pacLeft = getPacDasher().getXY().getX() - xPacCoreRadius;
        int pacRight = getPacDasher().getXY().getX() + xPacCoreRadius;
        int pacTop = getPacDasher().getXY().getY() - yPacCoreRadius;
        int pacBottom = getPacDasher().getXY().getY() + yPacCoreRadius;
        
        int xGCoreRadius = UIModelConsts.GHOST_DIAMETER/4;
        int yGCoreRadius = UIModelConsts.GHOST_DIAMETER/4;    
        
        for (int i=0; i<ghostList.size(); i++)
        {
            Ghost ghost = (Ghost) ghostList.get(i);
            int gLeft = ghost.getXY().getX() - xGCoreRadius;
            int gRight = ghost.getXY().getX() + xGCoreRadius;
            int gTop = ghost.getXY().getY() - yGCoreRadius;
            int gBottom = ghost.getXY().getY() + yGCoreRadius;
            //Output.print(Output.DEBUG, gLeft + "," + gRight + "," + 
            //    gTop + "," + gBottom + ":::" + pacLeft + "," +
            //    pacRight + "," + pacTop + "," + pacBottom);   
            if (gLeft < pacRight && gRight > pacLeft
                && gBottom > pacTop && gTop < pacBottom)
            {
                //Output.print(Output.DEBUG, "********** collision!!");
                fireMobileCollision(ghost);         
            }
        }
    }
    
    private void fireMobileCollision(Ghost ghost)
    {
        int ghostState = ghost.getState();
        //Output.print(Output.DEBUG, "collision, ghost state is " + ghostState);
        //Output.print(Output.DEBUG, "ghost posAndDirection " + ghost.getPosAndDirection());
        if (ghostState == GhostStrategy.FLIGHT_STATE)
        {
            getPacDasher().addToScore(ghost.getScoreValue());
            UIModel.setState(State.CAPTURE_STATE);
            ghost.reset();
        }
        else // normal case
        {
            getPacDasher().loseLife();
            resetMobiles();
            UIModel.setState(State.READY_STATE);
        }
    }
    
    private static void addToList(List list, XYCoarse[] array)
    {
        for (int i=0; i<array.length; i++)
        {
            if (array[i] == null)
            {
                break;
            }
            list.add((XYCoarse)array[i]);
        }
    }
    
    public List getAnimatedList()
    {
        return animatedList;
    }
    
    public List getGhostList()
    {
        return ghostList;
    }
    
    public Color getWallColor()
    {
        return wallColor;
    }        
        
    public GhostBlinky getGhostBlinky()
    {
        return ghostBlinky;
    }
    
    public GhostPinky getGhostPinky()
    {
        return ghostPinky;
    }
    
    public GhostInky getGhostInky()
    {
        return ghostInky;
    }
    
    public GhostClyde getGhostClyde()
    {
        return ghostClyde;
    }

    
    public int getWidth()
    {
        return width;
    }
    
    public int getHeight()
    {
        return height;
    }
    
    
    // utility method
    public static HashSet arrayInt2DToSet(int[][] array)
    {
         HashSet set = new HashSet();
         for (int i=0; i<array.length; i++)
         {
             int xCoarse = array[i][0];
             int yCoarse = array[i][1];
             if (XYCManager.isInitialized())
             {
                 set.add(XYCManager.getInstance().createXYC(xCoarse, yCoarse));
             }             
             else
             {
                 set.add(new XYCoarse(xCoarse, yCoarse));
             }
         }
         return set;
    }
    
    
} 