/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;

import com.oranda.pacdasher.uimodel.util.*;

import java.awt.Color;

/**
 * MazeFactory - factory for Maze objects
 */
 
public class MazeFactory
{
    public static Maze create(Direction pacDirection, int mazeID)
    {
        int[][] walls = new int[0][0];
        int[][] boxWalls = new int[0][0];
        int[][] clearArea = new int[0][0];
        int[][] clearAreaExtra = new int[0][0];
        int[][] gates = new int[0][0];
        int[][] pacDashers = new int[0][0];
        int[][] ghostsGeneric = new int[0][0];
        int[][] ghostsBlinky = new int[0][0];
        int[][] ghostsPinky = new int[0][0];
        int[][] ghostsInky = new int[0][0];
        int[][] ghostsClyde = new int[0][0];
        int[][] energizers = new int[0][0];

        Color wallColor = Color.BLUE;
        Color gateColor = Color.PINK;
                
        switch (mazeID)
        {
            case 1 :    
                wallColor = Color.BLUE;
                gateColor = Color.PINK;
                walls = new int[][] {
                {0,0},{1,0},{2,0},{3,0},{4,0},{5,0},{6,0},{7,0},
                  {8,0},{9,0},{10,0},{11,0},{12,0},{13,0},{14,0},{15,0},
                  {16,0},{17,0},{18,0},{19,0},{20,0},{21,0},{22,0},{23,0},
                  {24,0},{25,0},{26,0},{27,0},{28,0},
                {0,1},{14,1},{28,1},
                {0,2},{14,2},{28,2},
                {0,3},{3,3},{4,3},{5,3},
                  {8,3},{9,3},{10,3},{11,3},{14,3},
                  {17,3},{18,3},{19,3},{20,3},
                  {23,3},{24,3},{25,3},{28,3},
                {0,4},{3,4},{4,4},{5,4},
                  {8,4},{9,4},{10,4},{11,4},{14,4},
                  {17,4},{18,4},{19,4},{20,4},
                  {23,4},{24,4},{25,4},{28,4},
                {0,5},{28,5},
                {0,6},{28,6},  
                {0,7},{3,7},{4,7},{5,7},{8,7},
                  {11,7},{12,7},{13,7},{14,7},{15,7},{16,7},{17,7},
                  {20,7},{23,7},{24,7},{25,7},{28,7},
                {0,8},{8,8},{14,8},{20,8},{28,8},
                {0,9},{8,9},{14,9},{20,9},{28,9},
                {0,10},{1,10},{2,10},{3,10},{4,10},{5,10},
                  {8,10},{9,10},{10,10},{11,10},{14,10},
                  {17,10},{18,10},{19,10},{20,10},
                  {23,10},{24,10},{25,10},{26,10},{27,10},{28,10},
                {0,11},{1,11},{2,11},{3,11},{4,11},{5,11},
                  {8,11},{20,11},
                  {23,11},{24,11},{25,11},{26,11},{27,11},{28,11},
                {0,12},{1,12},{2,12},{3,12},{4,12},{5,12},
                  {8,12},{20,12},
                  {23,12},{24,12},{25,12},{26,12},{27,12},{28,12},
                {0,13},{1,13},{2,13},{3,13},{4,13},{5,13},
                  {8,13},{20,13},
                  {23,13},{24,13},{25,13},{26,13},{27,13},{28,13},
                {0,16},{1,16},{2,16},{3,16},{4,16},{5,16},
                  {8,16},{20,16},
                  {23,16},{24,16},{25,16},{26,16},{27,16},{28,16},
                {0,17},{1,17},{2,17},{3,17},{4,17},{5,17},
                  {8,17},{20,17},
                  {23,17},{24,17},{25,17},{26,17},{27,17},{28,17},
                {0,18},{1,18},{2,18},{3,18},{4,18},{5,18},
                  {8,18},{20,18},
                  {23,18},{24,18},{25,18},{26,18},{27,18},{28,18},
                {0,19},{1,19},{2,19},{3,19},{4,19},{5,19},
                  {8,19},
                  {11,19},{12,19},{13,19},{14,19},{15,19},{16,19},{17,19},
                  {20,19},
                  {23,19},{24,19},{25,19},{26,19},{27,19},{28,19},
                {0,20},{14,20},{28,20},
                {0,21},{14,21},{28,21},
                {0,22},{3,22},{4,22},{5,22},
                  {8,22},{9,22},{10,22},{11,22},{14,22},
                  {17,22},{18,22},{19,22},{20,22},
                  {23,22},{24,22},{25,22},{28,22},
                {0,23},{5,23},{23,23},{28,23},
                {0,24},{5,24},{23,24},{28,24},
                {0,25},{1,25},{2,25},{5,25},{8,25},
                  {11,25},{12,25},{13,25},{14,25},{15,25},{16,25},{17,25},
                  {20,25},{23,25},{26,25},{27,25},{28,25},
                {0,26},{8,26},{14,26},{20,26},{28,26},
                {0,27},{8,27},{14,27},{20,27},{28,27},
                {0,28},{3,28},{4,28},{5,28},{6,28},{7,28},{8,28},{9,28},{10,28},
                  {11,28},{14,28},{17,28},{18,28},{19,28},{20,28},{21,28},
                  {22,28},{23,28},{24,28},{25,28},{28,28},
                {0,29},{28,29},
                {0,30},{28,30},
                {0,31},{1,31},{2,31},{3,31},{4,31},{5,31},{6,31},{7,31},{8,31},
                  {9,31},{10,31},{11,31},{12,31},{13,31},{14,31},{15,31},{16,31},
                  {17,31},{18,31},{19,31},{20,31},{21,31},{22,31},{23,31},{24,31},
                  {25,31},{26,31},{27,31},{28,31}
                };                  
                
                  
            
                boxWalls = new int[][] {
                    {11,13},{12,13},{16,13},{17,13},
                    {11,14},{17,14},
                    {11,15},{17,15},
                    {11,16},{12,16},{13,16},{14,16},{15,16},{16,16},{17,16}
                };
                
                // no dots, boundaries
                clearArea = new int[][] {
                    {8,10}, {21,20}
                };
                
                clearAreaExtra = new int[][] {
                    {1,15},{2,15},{3,15},{4,15},{5,15},{6,15},
                    {23,15},{24,15},{25,15},{26,15},{27,15},{28,15}};
                    
                gates = new int[][] {
                    {13,13},{14,13},{15,13}
                };
                
                pacDashers = new int[][] {{UIModelConsts.PAC_DASHER_INIT_X,
                        UIModelConsts.PAC_DASHER_INIT_Y}};
                ghostsGeneric = new int[][] {};
                ghostsBlinky = new int[][] {{UIModelConsts.GHOST_BLINKY_INIT_X,
                        UIModelConsts.GHOST_BLINKY_INIT_Y}};
                ghostsPinky = new int[][] {{UIModelConsts.GHOST_PINKY_INIT_X,
                        UIModelConsts.GHOST_PINKY_INIT_Y}};
                ghostsInky = new int[][] {{UIModelConsts.GHOST_INKY_INIT_X,
                        UIModelConsts.GHOST_INKY_INIT_Y}};
                ghostsClyde = new int[][] {{UIModelConsts.GHOST_CLYDE_INIT_X,
                        UIModelConsts.GHOST_CLYDE_INIT_Y}};
                
                energizers = new int[][] {
                    {2,4},{27,4},{2,24},{27,24}
                };
            
                break;
            
            case 2 :    
                wallColor = Color.ORANGE;
                gateColor = Color.GREEN;    
                walls = new int[][] {
                {0,0},{1,0},{2,0},{3,0},{4,0},{5,0},{6,0},{7,0},
                  {8,0},{9,0},{10,0},{11,0},{12,0},{13,0},{14,0},{15,0},
                  {16,0},{17,0},{18,0},{19,0},{20,0},{21,0},{22,0},{23,0},
                  {24,0},{25,0},{26,0},{27,0},{28,0},
                {0,1},{8,1},{20,1},{28,1},
                {0,2},{8,2},{20,2},{28,2},
                {0,3},{3,3},{4,3},{5,3},{8,3},
                {11,3},{12,3},{13,3},{14,3},{15,3},{16,3},{17,3},
                  {20,3},{23,3},{24,3},{25,3},{28,3},
                {0,4},{28,4},
                {0,5},{28,5},
                {0,6},{1,6},{2,6},{5,6},{8,6},{9,6},{10,6},{11,6},{14,6},
                  {17,6},{18,6},{19,6},{20,6},{23,6},{26,6},{27,6},{28,6},
                {0,7},{1,7},{2,7},{5,7},{8,7},{9,7},{10,7},{11,7},{14,7},
                  {17,7},{18,7},{19,7},{20,7},{23,7},{26,7},{27,7},{28,7},
                {5,8},{14,8},{23,8},
                {5,9},{14,9},{23,9},
                {0,10},{1,10},{2,10},{5,10},{6,10},{7,10},{8,10},{11,10},
                  {12,10},{13,10},{14,10},{15,10},{16,10},{17,10},
                  {20,10},{21,10},{22,10},{23,10},{26,10},{27,10},{28,10},
                {0,11},{1,11},{2,11},{26,11},{27,11},{28,11},
                {0,12},{1,12},{2,12},{26,12},{27,12},{28,12},
                {0,13},{1,13},{2,13},{5,13},{6,13},{7,13},{8,13},
                  {20,13},{21,13},{22,13},{23,13},{26,13},{27,13},{28,13},
                {0,14},{1,14},{2,14},{5,14},{23,14},
                  {26,14},{27,14},{28,14},
                {0,15},{1,15},{2,15},{5,15},{23,15},
                  {26,15},{27,15},{28,15},
                {0,16},{1,16},{2,16},{5,16},{8,16},{20,16},
                  {23,16},{26,16},{27,16},{28,16},
                {8,17},{20,17},
                {8,18},{20,18},
                {0,19},{1,19},{2,19},{5,19},{6,19},{7,19},{8,19},{9,19},
                  {10,19},{11,19},{14,19},{17,19},{18,19},{19,19},{20,19},
                  {21,19},{22,19},{23,19},{26,19},{27,19},{28,19},
                {0,20},{1,20},{2,20},{14,20},{26,20},{27,20},{28,20},
                {0,21},{1,21},{2,21},{14,21},{26,21},{27,21},{28,21},
                {0,22},{1,22},{2,22},{5,22},{6,22},{7,22},{8,22},
                  {11,22},{12,22},{13,22},{14,22},{15,22},{16,22},{17,22},
                  {20,22},{21,22},{22,22},{23,22},{26,22},{27,22},{28,22},
                {0,23},{28,23},
                {0,24},{28,24},
                {0,25},{3,25},{4,25},{5,25},{8,25},{9,25},{10,25},{11,25},
                  {14,25},{17,25},{18,25},{19,25},{20,25},
                  {23,25},{24,25},{25,25},{28,25},
                {0,26},{3,26},{4,26},{5,26},{8,26},{14,26},{20,26},{23,26},
                  {24,26},{25,26},{28,26},
                {0,27},{3,27},{4,27},{5,27},{8,27},{14,27},{20,27},{23,27},
                  {24,27},{25,27},{28,27},
                {0,28},{3,28},{4,28},{5,28},{8,28},{11,28},{12,28},{13,28},
                  {14,28},{15,28},{16,28},{17,28},{20,28},{23,28},{24,28},
                  {25,28},{28,28},
                {0,29},{28,29},
                {0,30},{28,30},
                {0,31},{1,31},{2,31},{3,31},{4,31},{5,31},{6,31},{7,31},{8,31},
                  {9,31},{10,31},{11,31},{12,31},{13,31},{14,31},{15,31},{16,31},
                  {17,31},{18,31},{19,31},{20,31},{21,31},{22,31},{23,31},{24,31},
                  {25,31},{26,31},{27,31},{28,31}
                };
            
                boxWalls = new int[][] {
                    {11,13},{12,13},{16,13},{17,13},
                    {11,14},{17,14},
                    {11,15},{17,15},
                    {11,16},{12,16},{13,16},{14,16},{15,16},{16,16},{17,16}
                };
                
                // no dots, boundaries
                clearArea = new int[][] {
                    {5,9}, {24,19}
                };
                
                clearAreaExtra = new int[][] {
                    {13,20},{13,21},{12,21},{11,21},
                    {16,20},{16,21},{17,21},{18,21}};
                    
                gates = new int[][] {
                    {13,13},{14,13},{15,13}
                };
                
                pacDashers = new int[][] {{UIModelConsts.PAC_DASHER_INIT_X,
                        UIModelConsts.PAC_DASHER_INIT_Y}};
                ghostsGeneric = new int[][] {};
                ghostsBlinky = new int[][] {{UIModelConsts.GHOST_BLINKY_INIT_X,
                        UIModelConsts.GHOST_BLINKY_INIT_Y}};
                ghostsPinky = new int[][] {{UIModelConsts.GHOST_PINKY_INIT_X,
                        UIModelConsts.GHOST_PINKY_INIT_Y}};
                ghostsInky = new int[][] {{UIModelConsts.GHOST_INKY_INIT_X,
                        UIModelConsts.GHOST_INKY_INIT_Y}};
                ghostsClyde = new int[][] {{UIModelConsts.GHOST_CLYDE_INIT_X,
                        UIModelConsts.GHOST_CLYDE_INIT_Y}};
                
                energizers = new int[][] {
                    {2,3},{27,3},{2,28},{27,28}
                };
            
                break;
            
            default :
                break;
        }
        return new Maze(pacDirection, walls, boxWalls, 
            clearArea, clearAreaExtra, gates,
            energizers,
            pacDashers,  ghostsGeneric, ghostsBlinky,
            ghostsPinky, ghostsInky, ghostsClyde, 
            wallColor, gateColor);    
    }
}