/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;

import com.oranda.util.Output;
import com.oranda.pacdasher.ui.DirtyArea;
import com.oranda.pacdasher.uimodel.util.*;

import java.awt.Graphics;
import java.util.ArrayList;

public abstract class MobileVisualObject extends AnimatedVisualObject 
implements IMobile
{
    protected PosAndDirection posAndDirection;
    protected PosAndDirection initPosAndDirection;
    
    protected DirtyArea dirtyArea = new DirtyArea();    
    
    public MobileVisualObject(XYCoarse xyCoarse)
    {
        super(xyCoarse);
        //this.dirtyArea = new DirtyArea(xyCoarse);
        Direction direction = new Direction(DirectionCode.LEFT);
        this.posAndDirection 
            = new PosAndDirection(this, xyCoarse, direction);
        this.initPosAndDirection = (PosAndDirection) posAndDirection.clone();
    }

    public void reset()
    {
        this.posAndDirection = (PosAndDirection) initPosAndDirection.clone();    
    }
             
    public static int getDiameter() 
    {
        // default
        return UIModelConsts.X_PAC_DASHER_SIZE;
    }    
    
    public void render(Graphics g) {
        // should not be called
        Output.print(Output.DEBUG, "MobileVisualObject.render(g)");
    }
    
    public void render(Graphics g, XYCoarse xyCoarse)
    {
        setPos(xyCoarse);
        this.render(g);
    }
    
    /*public XYCoarse getXYCoarse()
    {
        return posAndDirection.getXYCoarse();        
    }*/

    public void setPos(XYCoarse xyCoarse)
    {
        posAndDirection.setPos(xyCoarse);
    }
    
    public void initPos(XYCoarse xyCoarse)
    {        
        posAndDirection.setPos(xyCoarse);
        this.initPosAndDirection = (PosAndDirection) posAndDirection.clone();
    }
    
    public XY getXY()
    {
        return posAndDirection.getXY();        
    }
    
    public XYCoarse getXYCoarse()
    {
        return posAndDirection.nearestXYCoarse();
    }
        
    public DirectionCode getDirectionCode()
    {
        return posAndDirection.getCurDirectionCode();
    }

    public boolean isMovingHorizontally()
    {
        return getDirectionCode() == DirectionCode.RIGHT
            || getDirectionCode() == DirectionCode.LEFT;
    }

    public boolean isMovingVertically()
    {
        return getDirectionCode() == DirectionCode.UP
            || getDirectionCode() == DirectionCode.DOWN;
    }    
    
    protected abstract void move();
    
    public void moveAndSetDirtyArea()
    {
        move();
        setDirtyArea();
    }
        
        
/*
    public void clip(Graphics g1, int x, int y)
    {
    }
*/
    public int getXTopLeft(XYCoarse xyCoarse)
    {        
        return (xyCoarse.getX() - 1) * (UIModelConsts.X_TILE_SIZE);
    }
    
    public int getYTopLeft(XYCoarse xyCoarse)
    {
        return (xyCoarse.getY() - 1) * (UIModelConsts.Y_TILE_SIZE);
    }
    
    // like Rectangle.contains() but Rectangles are garbage
    public boolean envelopes(int xLeft, int xRight, int yTop, int yBottom)
    {
        int thisXLeft = getXLeft();
        int thisXRight = getXRight();
        int thisYTop = getYTop();
        int thisYBottom = getYBottom();
        
        if (xLeft >= thisXLeft && xRight <= thisXRight
            && yTop >= thisYTop && yBottom <= thisYBottom)
        {
            return true;
        }
        return false;
    }
    
    public int getXLeft()
    {
        return getXY().getX() - getDiameter()/2;
    }
    
    public int getXRight()
    {
        return getXY().getX() + getDiameter()/2;
    }
    
    public int getYTop()
    {
        return getXY().getY() - getDiameter()/2;
    }
    
    public int getYBottom()
    {
        return getXY().getY() + getDiameter()/2;
    }


    public void setDirtyArea()
    {
    	//this.dirtyArea.addMVOArea(this);
    	
    	

        XYCManager xycm = XYCManager.getInstance();
        XYCoarse xyCoarse = getXYCoarse();
        int xCoarse = xyCoarse.getX();
        int yCoarse = xyCoarse.getY();
        XYCoarse xycLeftTop = null;
        XYCoarse xycRightBottom = null;

        xycLeftTop = xycm.createXYC(xCoarse - 2, yCoarse - 2);
        xycRightBottom = xycm.createXYC(xCoarse + 2, yCoarse + 2);
              
        /*
        DirectionCode dirCode = getDirectionCode();
        if (dirCode == DirectionCode.LEFT)
        {
            xycLeftTop = xycm.createXYC(xCoarse - 1, yCoarse - 1);
            xycRightBottom = xycm.createXYC(xCoarse + 2, yCoarse);
        }
        else if (dirCode == DirectionCode.RIGHT)
        {
            xycLeftTop = xycm.createXYC(xCoarse - 2, yCoarse - 1);
            xycRightBottom = xycm.createXYC(xCoarse + 1, yCoarse);
        
        }
        else if (dirCode == DirectionCode.UP)
        {
            xycLeftTop = xycm.createXYC(xCoarse - 1, yCoarse - 1);
            xycRightBottom = xycm.createXYC(xCoarse, yCoarse + 2);
        
        }
        else if (dirCode == DirectionCode.DOWN)
        {
            xycLeftTop = xycm.createXYC(xCoarse - 1, yCoarse - 2);
            xycRightBottom = xycm.createXYC(xCoarse, yCoarse + 1);
        
    	}
    	*/
        this.dirtyArea.set(xycLeftTop, xycRightBottom);
        
    }
    
    
    public DirtyArea getDirtyArea()
    {
        if (dirtyArea == null)
        {
            setDirtyArea();
        }
        return dirtyArea;
    }
    
}    