/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package com.oranda.pacdasher.uimodel;

import com.oranda.pacdasher.uimodel.event.IPacListener;
import com.oranda.pacdasher.uimodel.event.PacEvent;
import com.oranda.pacdasher.uimodel.event.PacScoreEvent;
import com.oranda.pacdasher.uimodel.event.PacMoveEvent;
import com.oranda.pacdasher.uimodel.util.*;
import com.oranda.util.Output;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import javax.imageio.ImageIO;

/**
 * Representing the PacDasher sprite 
 */
public class PacDasher extends MobileVisualObject implements IPlayer
{
    protected BufferedImage iPacDasher, iPacright, iPacleft, iPacup, iPacdown;
    protected Image curImg;
    
    protected int score = 0;        
    private static int highScore = UIModelConsts.HIGH_SCORE_INIT;;
    
    protected static final String imagesPath = UIModelConsts.PATH_IMAGES;
    protected int numLives = UIModelConsts.NUM_LIVES_INIT;
    
    ArrayList iPacListeners = new ArrayList(3);
    
    public PacDasher(XYCoarse xyCoarseInit)
    {
        super(xyCoarseInit);
        setDirectionCode(DirectionCode.LEFT);
        
        File fPacDasher = new File("");
        File fPacright = new File("");
        File fPacleft = new File("");
        File fPacup = new File("");
        File fPacdown = new File("");
        try
        {
            fPacDasher = new File(imagesPath + "pacDasher.gif");
            iPacDasher = (BufferedImage) ImageIO.read(fPacDasher);
            fPacright = new File(imagesPath + "pacRight.gif");
            iPacright = (BufferedImage) ImageIO.read(fPacright);
            fPacleft = new File(imagesPath + "pacLeft.gif");
            iPacleft = (BufferedImage) ImageIO.read(fPacleft);
            fPacup = new File(imagesPath + "pacUp.gif");
            iPacup = (BufferedImage) ImageIO.read(fPacup);
            fPacdown = new File(imagesPath + "pacDown.gif");
            iPacdown = (BufferedImage) ImageIO.read(fPacdown);
            curImg = iPacleft;
            // announce initial position
            fireMoved(posAndDirection.getXY());          
        } 
        catch (IOException ioe)
        {
            Output.print(Output.DEBUG, "Error loading image files for PacDasher,  "
                + "e.g. " + fPacDasher.getAbsolutePath(), ioe);
        }
        
    }

    public void resetForGame()
    {
        super.reset();
        setScore(0);
        setNumLives(UIModelConsts.NUM_LIVES_INIT);    
    }
    
    public DirectionCode getDirectionCode()
    {
        return posAndDirection.getCurDirectionCode();
    }
    
    public void addPacListener(IPacListener IPacListener)
    {
        iPacListeners.add(IPacListener);
    }
    /*
    public void clip(Graphics g1, Graphics g2, XYCoarse xyCoarse)
    {
        //paint(g);
        clip(g1, xyCoarse.getX(), xyCoarse.getY());
        clip(g2, xyCoarse.getX(), xyCoarse.getY());
    }
    */
    public void clip(Graphics g, int x, int y)
    {
        g.setClip(null);
        g.clipRect(x - 5, y - 5, UIModelConsts.X_PAC_DASHER_SIZE + 10, 
            UIModelConsts.Y_PAC_DASHER_SIZE + 10);
    }
    

    public void render(Graphics g, XYCoarse XYCoarse) {};
    
    public static int getDiameter() 
    {
        return UIModelConsts.X_PAC_DASHER_SIZE;
    }
                
    public int getScore()
    {
        return score;
    }
                
    public int getHighScore()
    {
        return highScore;
    }
    
    public int getNumLives()
    {
        return numLives;
    }
    

    public void render(Graphics g)
    {
        DirectionCode directionCode = posAndDirection.getCurDirectionCode();
        int movesPerTile = UIModelConsts.X_TILE_SIZE/UIModelConsts.MOVE_SIZE;
        if (UIModel.getInstance().getVirtualTime()%movesPerTile 
            < movesPerTile/2 || directionCode == DirectionCode.STATIONARY)
        {
            // mouth open            
            Image img = getAppropriateImage(directionCode);
            if (img != null)
            {
                curImg = img;
            }                
        } 
        else
        { 
            // mouth not open
            curImg = iPacDasher;
        } 

        int xTopCorner = posAndDirection.getX() - UIModelConsts.X_TILE_SIZE;
        int yTopCorner = posAndDirection.getY() - UIModelConsts.Y_TILE_SIZE;
        //clip(g, xTopCorner, yTopCorner);
        g.drawImage(curImg, xTopCorner + 1, yTopCorner + 1, 
            UIModelConsts.X_PAC_DASHER_SIZE, 
            UIModelConsts.Y_PAC_DASHER_SIZE, null);
    }

    
    
    public Image getAppropriateImage(DirectionCode directionCode)
    {
        if (directionCode == DirectionCode.DOWN)
        {
            return iPacdown;
        } 
        else if (directionCode == DirectionCode.UP)
        {
            return iPacup;
        } 
        else if (directionCode == DirectionCode.LEFT)
        {
            return iPacleft;
        } 
        else if (directionCode == DirectionCode.RIGHT)
        {
            return iPacright;
        } 
        else
        {
            return null;
        }                
    }    
    
    public PosAndDirection getPosAndDirection()
    {
        return posAndDirection;
    }
    
    public void move()
    {
        XY xy = posAndDirection.move();
        if (getDirectionCode() != DirectionCode.STATIONARY)
        {
            fireMoved(xy);
        }
    }    
    
    public void addToScore(int scoreValue)
    {
        //Output.print(Output.DEBUG, "addToScore " + scoreValue);
        setScore(this.score + scoreValue);
        
    }
    
    public void setScore(int score)
    {
        this.score = score;
        if (score > highScore)
        {
            this.highScore = score;
        }
        fireScoreChanged();
    }
    
    public void setNumLives(int numLives)
    {
        this.numLives = numLives;
        fireLivesChanged();
        UIModel.getInstance().resetVirtualTime();
    }
    
    public void gainedPower()
    {
        fireGainedPower();
    }
    
    public void loseLife()
    {
        //Output.print(Output.DEBUG, "loseLife");
        setNumLives(this.numLives - 1);
        //Output.print(Output.DEBUG, "numLives " + numLives);
    }
    
    
    private void fireScoreChanged()
    {
        for (int i = 0; i < iPacListeners.size(); i++) 
        {
            //Output.print(Output.DEBUG, "fireScore " + score);
            IPacListener listener = (IPacListener) iPacListeners.get(i);
            listener.pacScoreChanged(new PacScoreEvent(this));
        }
    }        
    
    private void fireMoved(XY xy) 
    {
        // assume that listeners do not deregister themselves during
        // the execution of this method
        for (int i = 0; i < iPacListeners.size(); i++) 
        {
            IPacListener listener = (IPacListener) iPacListeners.get(i);
            listener.pacDasherMoved(PacMoveEvent.getInstance(xy, 
                posAndDirection.getCurDirectionCode(),this));
        }
    }
    
    private void fireGainedPower() 
    {

        // assume that listeners do not deregister themselves during
        // the execution of this method
        for (int i = 0; i < iPacListeners.size(); i++) 
        {
            IPacListener listener = (IPacListener) iPacListeners.get(i);
            listener.pacDasherGainedPower();
        }
    }
    
    private void fireLivesChanged() 
    {
        // assume that listeners do not deregister themselves during
        // the execution of this method
        for (int i = 0; i < iPacListeners.size(); i++) 
        {
            IPacListener listener = (IPacListener) iPacListeners.get(i);
            listener.pacDasherLivesChanged(new PacEvent(this));
        }
    }

    public void setDirectionCode(DirectionCode directionCode)
    {
        posAndDirection.setDesiredDirectionCode(directionCode);
    }        
}