/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;

import com.oranda.pacdasher.uimodel.util.*;
import com.oranda.util.Output;

import java.util.HashSet;

/**
 * PosAndDirection
 * 
 */
 
public class PosAndDirection
{
    /*
     * There are two systems of coordinates:
     * one nearer to the graphics, pixel-based: x,y 
     * and one nearer to the model, tile-based: 
     * xCoarse, and yCoarse
     */
    private XY xy = new XY(-1, -1);
    private XYCoarse xyCoarse = new XYCoarse(-1, -1);
    
    // save last x and y matching exactly a coarse coordinate
    //private XY xyLast;
    
    private Direction direction;

    private static int increment = UIModelConsts.MOVE_SIZE;
    
    private MobileVisualObject mvo;
        
    public PosAndDirection(MobileVisualObject mvo,
        XYCoarse xyCoarseInit, Direction direction)
    throws IllegalArgumentException
    {
        setPos(xyCoarseInit);
        if (direction == null)
        {
            throw new IllegalArgumentException("direction");
        }
        this.direction = direction;    
        this.mvo = mvo;
    }
    
    public Object clone()
    {
        return new PosAndDirection(mvo, xyCoarse, direction);
    }
    
    public void setPos(XYCoarse xyCoarse)
    {
        this.xyCoarse = xyCoarse;
        this.xy = translateXY(xyCoarse);

        //xLast = x;
        //yLast = y;
    }
    
    public XY translateXY(XYCoarse xyCoarse)
    {
        int x = translateX(xyCoarse.getX());
        int y = translateY(xyCoarse.getY());
        return new XY(x, y);
    }
    
    public int translateX(int xCoarse)
    {
        return xCoarse * UIModelConsts.X_TILE_SIZE;
    }
    public int translateY(int yCoarse)
    {
        return yCoarse * UIModelConsts.Y_TILE_SIZE;
    }    
        
    public int translateXCoarse(int x)
    {
        return x/UIModelConsts.X_TILE_SIZE;
    }
    
    public int translateYCoarse(int y)
    {
        return y/UIModelConsts.Y_TILE_SIZE;
    }
    
    public int nearestXCoarse()
    {
        return translateXCoarse(getX());
    }
    
    public int nearestYCoarse()
    {
        return translateYCoarse(getY());
    }
    
    public XYCoarse nearestXYCoarse()
    {
        return XYCManager.getInstance().createXYC(nearestXCoarse(), nearestYCoarse());
    }
    
    XY move()
    {
        //Output.print(Output.DEBUG, "Trying to move 1st " + direction.getDirectionCode());
        if (move(direction.getDesiredDirectionCode()))
        {
            //Output.print(Output.DEBUG, "curDirectionCode " + direction.getCurDirectionCode());
            direction.setCurDirectionToDesired();
        }
        else if (move(direction.getCurDirectionCode()))
        {
            //Output.print(Output.DEBUG, "curDirectionCode");
        }
        else
        {

        }
        return xy;
    }
    
    boolean move(DirectionCode directionCode)
    {
        //Output.print(Output.DEBUG, "xCoarse " + xCoarse 
        //    + ", yCoarse " + yCoarse);
        
        int xNext, yNext;
        
        if (directionCode == DirectionCode.RIGHT)
        { 
            if (!isRightBlocked())
            {
                // first implement wraparound
                int width = UIModel.getInstance().getMaze().getWidth();
                if (getX() > width + UIModelConsts.X_PAC_DASHER_SIZE)
                {
                    setX(- UIModelConsts.X_PAC_DASHER_SIZE);
                }
                
                // mainline
                setX(getX() + increment);

                //xNext = xLast + UIModelConsts.X_TILE_SIZE;
                //if (x == xNext)
                //{
                //    xCoarse++;
                //    xLast = xNext;
                //}
                return true;
            }

        }
        else if (directionCode == DirectionCode.LEFT)
        {
            if (!isLeftBlocked())
            {
                // first implement wraparound
                int width = UIModel.getInstance().getMaze().getWidth();
                if (getX() < 0 - UIModelConsts.X_PAC_DASHER_SIZE)
                {
                    setX(width);
                }
                
                // main line
                setX(getX() - increment);
                //xNext = xLast - UIModelConsts.X_TILE_SIZE;
                //if (x == xNext)
                //{
                //    xCoarse--;
                //    xLast = xNext;
                //}
                return true;
            }
        }
        else if (directionCode == DirectionCode.DOWN)
        {
            if (!isDownBlocked())
            {
                setY(getY() + increment);
                //yNext = yLast + UIModelConsts.Y_TILE_SIZE;
                //if (y == yNext)
                //{
                //    yCoarse++;
                //    yLast = yNext;
                //}
                return true;
            }
        }
        else if (directionCode == DirectionCode.UP)
        {
            if (!isUpBlocked())
            {
                setY(getY() - increment);
                //yNext = yLast - UIModelConsts.Y_TILE_SIZE;
                //if (y == yNext)
                //{
                //    yCoarse--;
                //    yLast = yNext;
                //}
                return true;
            }
        }
        return false;
    }

    public boolean isStationary()
    {
        return (direction.getCurDirectionCode() == DirectionCode.STATIONARY);
    }
    
    public DirectionCode getCurDirectionCode()
    {
        return direction.getCurDirectionCode();
    }
    
    public DirectionCode getDesiredDirectionCode()
    {
        return direction.getDesiredDirectionCode();
    }
    
    public DirectionCode getPrevDirectionCode()
    {
        return direction.getPrevDirectionCode();
    }
    
    public void setDesiredDirectionCode(DirectionCode desiredDirectionCode)
    {
        direction.setDesiredDirectionCode(desiredDirectionCode);
    }
    
    public Direction getDirection()
    {
        return direction;
    }
    
    public DirectionCode getDirectionCode()
    {
        return direction.getCurDirectionCode();
    }

    public int getX()
    {
        return xy.getX();
    }
    
    public int getY()
    {
        return xy.getY();
    }
    
    public void setX(int x)
    {
        xy.setX(x);
    }
    
    public void setY(int y)
    {
        xy.setY(y);
    }
    
    /*
    public XYCoarse getXYCoarse()
    {
        return xyCoarse;
    }*/
    
    public XY getXY()
    {
        return xy;
    }
    
    /*
     * These methods check if the PacDasher is blocked in
     * the direction currently requested. If PacDasher is
     * exactly placed on a square in the coarse coordinate
     * system, it is only necessary to check one other square;
     * however, if PacDasher is between tiles (more likely),
     * it's necessary to check two squares - hence the extraCondition.
     */
    
    public boolean isRightBlocked()
    {
        return isXBlocked(getX() + UIModelConsts.X_TILE_SIZE, getY());
    }

    public boolean isLeftBlocked()
    {
        return isXBlocked(getX() - 2 * UIModelConsts.X_TILE_SIZE, getY());
    }

    public boolean isDownBlocked()
    {
        return isYBlocked(getX(), getY() + UIModelConsts.Y_TILE_SIZE);
    }

    public boolean isUpBlocked()
    {
        return isYBlocked(getX(), getY() - 2 * UIModelConsts.Y_TILE_SIZE);
    }            
    
    public boolean isXBlocked(int xPossibleWall, int y)
    {
        int xCoarse = translateXCoarse(xPossibleWall);
        int yCoarse = translateYCoarse(y);
            
        if (xPossibleWall%UIModelConsts.X_TILE_SIZE >= increment)
        {
            // walls only occur in multiples of X_TILE_SIZE
            return false;
        }
        else
        {

            
            
            // check if two or three squares to the side are blocked
            // depending on whether PacDasher is between tiles
            boolean extraCondition = false;
            if (y%UIModelConsts.Y_TILE_SIZE >= increment)
            {
                extraCondition = isBlocked(xCoarse, yCoarse + 1);
            }
            
            //Output.print(Output.DEBUG, "isXblocked? " + xCoarse + "," + yCoarse
            //    + isBlocked(xCoarse, yCoarse - 1) 
            //   + isBlocked(xCoarse, yCoarse)
            //   + extraCondition);
            return isBlocked(xCoarse, yCoarse - 1) 
                || isBlocked(xCoarse, yCoarse) || extraCondition;
        }
    }        
    
    public boolean isYBlocked(int x, int yPossibleWall)
    {
        int xCoarse = translateXCoarse(x);
        int yCoarse = translateYCoarse(yPossibleWall);
            
        int mazeWidth = UIModel.getInstance().getMaze().getWidth();
        if (x > mazeWidth || x < 0)
        {
            //Output.print(Output.DEBUG, "isYBlocked? " + mazeWidth + " " + x);

            // boundary case (for horizontal wraparound)
            return true;
        }
        if (yPossibleWall%UIModelConsts.Y_TILE_SIZE >= increment)
        {
            // walls only occur in multiples of Y_TILE_SIZE
            return false;
        }
        else
        {

            // check if two or three squares vertically are blocked
            // depending on whether PacDasher is between tiles
            boolean extraCondition = false;
            if (x%UIModelConsts.X_TILE_SIZE >= increment)
            {
                extraCondition = isBlocked(xCoarse + 1, yCoarse);
            }
                
            return isBlocked(xCoarse - 1, yCoarse) 
                || isBlocked(xCoarse, yCoarse) || extraCondition;
        }
    }            
    
    public boolean isBlocked(int xCoarse, int yCoarse)
    {    
        boolean isBlocked 
            = UIModel.getInstance().getMaze().isBlocked(mvo, xCoarse, yCoarse);
        //Output.print(Output.DEBUG, "isBlocked " + isBlocked + " "
        //    + xCoarse + " " + yCoarse);
        return isBlocked;
    }
    
    public String toString()
    {
        return xy + " " + direction;
    }
} 