/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;

import com.oranda.pacdasher.State;
import com.oranda.pacdasher.uimodel.util.*;
import com.oranda.util.Output;

import java.util.List;

/**
 * The UIModel contains the existence and positions
 * of all visual objects in the Maze: PacDasher, ghosts, 
 * dots, etc. 
 * Singleton.
 */ 
public class UIModel
{
    // the one true instance
    protected static UIModel uiModel;
    
    protected Maze maze;
    protected int level = 0;
    
    //protected int xOffset;
    //protected int yOffset;
    
    protected static int virtualTime; // starts at 0
    protected static Direction pacDirection;
    
    private PacDasher pacDasher;

        
    /*
     * Constructor is private because it's a singleton
     */
    private UIModel()
    {                        
    }
    
    public boolean isMazeFinished()
    {
        return maze.isMazeFinished();
    }
    
    public boolean isGameOver()
    {
        return pacDasher.getNumLives() == 0;
    }
        
    public void initialize()
    {
        XYCManager.initialize(UIModelConsts.MAZE_WIDTH + 1, 
            UIModelConsts.MAZE_HEIGHT + 1);
        this.pacDasher = new PacDasher(new XYCoarse(0,0));    
    }
    
    public PacDasher getPacDasher()
    {
        return this.pacDasher;
    }
    
    public void startNewGame()
    {
        level = 0;
        pacDasher.resetForGame();
        goToNewLevel();            
    }
    
    public void goToNewLevel()
    {
        level++;
        int mazeID = 2 - level%2;
        
        maze = MazeFactory.create(this.pacDirection, mazeID);
        
        List ghostList = maze.getGhostList();
        for (int i=0; i<ghostList.size(); i++)
        {
            Ghost ghost = (Ghost) ghostList.get(i);
            pacDasher.addPacListener(ghost);
        }
    }
    
    public void move()
    {
        maze.move();
        virtualTime++;
    }
    
    public static UIModel getInstance()
    {
        if (UIModel.uiModel == null) 
        {
            Output.print(Output.DEBUG, "new UIModel()");
            UIModel.uiModel = new UIModel();
        }
        return uiModel;
    }
    
        
    public Maze getMaze()
    {
        return maze;
    }
    
    public static int getVirtualTime()
    {
        return virtualTime;
    }
    
    public static void resetVirtualTime()
    {
        virtualTime = 0;
    }
        
    public void setDirectionCode(DirectionCode directionCode)
    {
        pacDasher.setDirectionCode(directionCode);
    }
    
    public static void setState(int state)
    {
        State.setState(state);
    }
} 