/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;

import com.oranda.pacdasher.uimodel.util.*;
import com.oranda.util.Output;

import java.awt.Graphics;
import java.lang.reflect.Constructor;
import java.util.List;

/*
 * A VisualObject is something which is initialized with a
 * coarse xy coordinate, like a sprite.
 */
public abstract class VisualObject
{    
    public static int getDiameter()
    {
        // default
        return UIModelConsts.X_TILE_SIZE;
    }
        
    public abstract void render(Graphics g, XYCoarse xyCoarse);

    public String toString()
    {
        return getClass().getName();
    }
    
    // classes should override if isNeighborly is true
    public void render(Graphics g, XYCoarse xyCoarse, 
        List neighbors)
    {
        // default
        render(g, xyCoarse);
    }
    
    // select classes may override
    public boolean isNeighborly()
    {
        return false;
    }
    
    protected VisualObject() {}
    
    public static int getX(XYCoarse xyCoarse)
    {
        return xyCoarse.getX() * UIModelConsts.X_TILE_SIZE ;
    }
    
    public static int getY(XYCoarse xyCoarse)
    {
        return xyCoarse.getY() * UIModelConsts.Y_TILE_SIZE;
    }
    
    public static int getXTopLeft(XYCoarse xyCoarse, int diameter)
    {
        return getX(xyCoarse) - diameter/2;
    }
    
    public static int getYTopLeft(XYCoarse xyCoarse, int diameter)
    {
        return getY(xyCoarse) - diameter/2;
    }

    // select classes may override
    public int getScoreValue()        
    {
        return 0;
    }

    /*
     * Whether to render on the intersection of grid lines
     * or on a block.
     */
    public boolean isIntersectionBased()
    {
        // default
        return false;
    }

}