/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel;

import com.oranda.pacdasher.uimodel.util.*;
import com.oranda.util.Output;

import java.awt.Color;
import java.awt.Graphics;
import java.util.List;

public class Wall extends VisualObject
{
    protected Color color = Color.BLUE;
    
    protected static int xTileSize = UIModelConsts.X_TILE_SIZE;
    protected static int yTileSize = UIModelConsts.Y_TILE_SIZE;
    protected static int diameter = xTileSize;
    
    public Wall(Color color)
    {
        this.color = color;
    }
                
    public void render(Graphics g, XYCoarse xyCoarse)
    {
        g.setColor(color);
        g.fillRect(getXTopLeft(xyCoarse, 0), 
            getYTopLeft(xyCoarse, 0), 
            diameter, diameter);
    }
    

    public void render(Graphics g, XYCoarse xyCoarse, List neighbors)
    {
        g.setColor(color);
        //Output.print(Output.DEBUG, "xyCoarse " + xyCoarse + ", neighbors" + neighbors);

        if (neighbors.size() == 4) 
        {
            //render(g, xyCoarse);    
        }
        else if (neighbors.contains(DirectionCode.UP) 
            && neighbors.contains(DirectionCode.DOWN)
            && neighbors.contains(DirectionCode.RIGHT))
        {
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize);        
        }
        else if (neighbors.contains(DirectionCode.UP)
            && neighbors.contains(DirectionCode.DOWN) 
            && neighbors.contains(DirectionCode.LEFT))
        {
            g.drawLine(getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0) + yTileSize);
        }
        else if (neighbors.contains(DirectionCode.RIGHT)
            && neighbors.contains(DirectionCode.LEFT) 
            && neighbors.contains(DirectionCode.DOWN))
        {
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + xTileSize,
                getYTopLeft(xyCoarse, 0));
        }
        else if (neighbors.contains(DirectionCode.RIGHT)
            && neighbors.contains(DirectionCode.UP) 
            && neighbors.contains(DirectionCode.LEFT))
        {
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize - 1,
                getXTopLeft(xyCoarse, 0) + xTileSize,
                getYTopLeft(xyCoarse, 0) + yTileSize - 1);    
        } 
        else if (neighbors.size() == 2 
            && neighbors.contains(DirectionCode.RIGHT) 
            && neighbors.contains(DirectionCode.UP))
        {
            g.drawArc(getXTopLeft(xyCoarse, 0),                          
                getYTopLeft(xyCoarse, 0),                             
                xTileSize,
                yTileSize,
                180,                             
                90);
                
            // across
            g.drawLine(getXTopLeft(xyCoarse, 0) + xTileSize/2 - 1,
                getYTopLeft(xyCoarse, 0) + yTileSize - 1,
                getXTopLeft(xyCoarse, 0) + xTileSize,
                getYTopLeft(xyCoarse, 0) + yTileSize - 1);    
                
            // vertical
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize/2 + 1);
        }
        else if (neighbors.size() == 2 
            && neighbors.contains(DirectionCode.RIGHT) 
            && neighbors.contains(DirectionCode.DOWN))
        {
            g.drawArc(getXTopLeft(xyCoarse, 0),                          
                getYTopLeft(xyCoarse, 0),                             
                xTileSize,
                yTileSize,
                90,                             
                90);
                
            // across
            g.drawLine(getXTopLeft(xyCoarse, 0) + xTileSize/2,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + xTileSize,
                getYTopLeft(xyCoarse, 0));    
                
            // vertical
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize/2,
                getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize);
        }
        else if (neighbors.size() == 2 
            && neighbors.contains(DirectionCode.LEFT) 
            && neighbors.contains(DirectionCode.UP))
        {
            g.drawArc(getXTopLeft(xyCoarse, 0),                          
                getYTopLeft(xyCoarse, 0),                             
                xTileSize,
                yTileSize,
                270,                             
                90);
                
            // across
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize - 1,
                getXTopLeft(xyCoarse, 0) + xTileSize/2 + 1,
                getYTopLeft(xyCoarse, 0) + yTileSize - 1);    
                
            // vertical
            g.drawLine(getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0) + yTileSize/2 + 1);
                
        }
        else if (neighbors.size() == 2 
            && neighbors.contains(DirectionCode.LEFT) 
            && neighbors.contains(DirectionCode.DOWN))
        {
            g.drawArc(getXTopLeft(xyCoarse, 0),                          
                getYTopLeft(xyCoarse, 0),                             
                xTileSize,
                yTileSize,
                0,                             
                90);
                
            // across
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + xTileSize/2,
                getYTopLeft(xyCoarse, 0));    
                
            // down
            g.drawLine(getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0)  + yTileSize/2 - 1,
                getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0) + yTileSize);            
        }
        else if (neighbors.contains(DirectionCode.RIGHT) 
            && neighbors.contains(DirectionCode.LEFT))
        {
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + xTileSize,
                getYTopLeft(xyCoarse, 0));
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize - 1,
                getXTopLeft(xyCoarse, 0) + xTileSize,
                getYTopLeft(xyCoarse, 0) + yTileSize - 1);
        }
        else if (neighbors.contains(DirectionCode.UP) 
            && neighbors.contains(DirectionCode.DOWN))
        {
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize);
            g.drawLine(getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0) + yTileSize);
        }
        else if (neighbors.size() == 1
            && neighbors.contains(DirectionCode.RIGHT))
        {
            g.drawArc(getXTopLeft(xyCoarse, 0),                          
                getYTopLeft(xyCoarse, 0),                             
                xTileSize,
                yTileSize,
                90,                             
                180);
            g.drawLine(getXTopLeft(xyCoarse, 0) + xTileSize/2 - 1,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + xTileSize,
                getYTopLeft(xyCoarse, 0));
            g.drawLine(getXTopLeft(xyCoarse, 0) + xTileSize/2 - 1,
                getYTopLeft(xyCoarse, 0) + yTileSize - 1,
                getXTopLeft(xyCoarse, 0) + xTileSize,
                getYTopLeft(xyCoarse, 0) + yTileSize - 1);        
        }
        else if (neighbors.contains(DirectionCode.LEFT))
        {
            g.drawArc(getXTopLeft(xyCoarse, 0) - 2,                          
                getYTopLeft(xyCoarse, 0),                             
                xTileSize,
                yTileSize,
                90,                             
                -180);
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + xTileSize/2,
                getYTopLeft(xyCoarse, 0));
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize - 1,
                getXTopLeft(xyCoarse, 0) + xTileSize/2,
                getYTopLeft(xyCoarse, 0) + yTileSize - 1);    
        }
        else if (neighbors.contains(DirectionCode.UP))
        {
            g.drawArc(getXTopLeft(xyCoarse, 0),                          
                getYTopLeft(xyCoarse, 0) - 2,                             
                xTileSize,
                yTileSize,
                0,                             
                -180);
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize/2);
            g.drawLine(getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0),
                getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0) + yTileSize/2);
        }
        else if (neighbors.contains(DirectionCode.DOWN))
        {
            g.drawArc(getXTopLeft(xyCoarse, 0),                          
                getYTopLeft(xyCoarse, 0),                             
                xTileSize,
                yTileSize,
                180,                             
                -180);
            g.drawLine(getXTopLeft(xyCoarse, 0)  + xTileSize - 1,
                getYTopLeft(xyCoarse, 0) + yTileSize/2 - 1,
                getXTopLeft(xyCoarse, 0) + xTileSize - 1,
                getYTopLeft(xyCoarse, 0) + yTileSize);
            g.drawLine(getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0)  + yTileSize/2 - 1,
                getXTopLeft(xyCoarse, 0),
                getYTopLeft(xyCoarse, 0) + yTileSize);
        }    
        else
        {
            // do nothing
            //render(g, xyCoarse);
        }
    
    }    
    
    public static int getDiameter()
    {
        return diameter;
    }

    public boolean isNeighborly()        
    {
        return true;
    }

}