/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel.util;

import java.awt.event.KeyEvent;


/* 
 * This class keeps track of the current direction
 * and desired direction, which depends on the state of 
 * the keys.
 */
public class Direction
{
    DirectionCode curDirectionCode;
    DirectionCode desiredDirectionCode;
    DirectionCode prevDirectionCode;
    
    public Direction()
    {
        // default
        this(DirectionCode.LEFT);
    }
    
    public Direction(DirectionCode directionCode)
    {
        this.curDirectionCode = directionCode;
    }    
    
    public DirectionCode getCurDirectionCode()
    {
        return curDirectionCode;
    }
    
    public DirectionCode getDesiredDirectionCode()
    {
        return desiredDirectionCode;
    }

    public DirectionCode getPrevDirectionCode()
    {
        return prevDirectionCode;
    }
    
    public void setDesiredDirectionCode(DirectionCode desiredDirectionCode)
    {
        this.desiredDirectionCode = desiredDirectionCode;
    }
    
    
    public void setCurDirectionToDesired()
    {
        if (!curDirectionCode.equals(desiredDirectionCode))
        {
            prevDirectionCode = (DirectionCode) curDirectionCode.clone();
            curDirectionCode = (DirectionCode) desiredDirectionCode;
        }
    }
    
        
    public static DirectionCode getDirectionCode(int keyCode) 
        throws IllegalArgumentException
    {
        switch (keyCode)
        {
            case KeyEvent.VK_DOWN:
                return DirectionCode.DOWN;
            case KeyEvent.VK_UP:
                return DirectionCode.UP;
            case KeyEvent.VK_LEFT:
                return DirectionCode.LEFT;
            case KeyEvent.VK_RIGHT:
                return DirectionCode.RIGHT;
            default:
                return DirectionCode.STATIONARY;
        }
    }    

    public Object clone()
    {
        return new Direction(this.curDirectionCode);
    }         
}