/**
 *  PacDasher application. For explanation of this class, see below. 
 *  Copyright (c) 2003 James McCabe. Email: jamesa@oranda.com 
 *  http://www.oranda.com/java/PacDasher/
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package com.oranda.pacdasher.uimodel.util;


import com.oranda.util.Output;

/**
 * XYCManager - overall model for the UI, to which the UI listens
 * Singleton.
 */
 
public class XYCManager
{
    // the one true instance
    protected static XYCManager xycManager;
    protected static XYCoarse[][] xycArray;
    protected int xCoarseBoundary;
    protected int yCoarseBoundary;
    
    /*
     * Constructor is private because it's a singleton
     */
    private XYCManager()
    {                        
    }

    private XYCManager(int xCoarseBoundary, int yCoarseBoundary) 
    {
        Output.print(Output.DEBUG, "XYCManager boundaries " + xCoarseBoundary + 
            "," + yCoarseBoundary);
        this.xCoarseBoundary = xCoarseBoundary;
        this.yCoarseBoundary = yCoarseBoundary;
        xycArray = new XYCoarse[xCoarseBoundary+12][yCoarseBoundary+4];
        for (int i=-4; i<xCoarseBoundary+8; i++)
        {
            for (int j=-2; j<yCoarseBoundary+2; j++)
            {
                XYCoarse xyCoarse = new XYCoarse(i, j);
                xycArray[i+4][j+2] = xyCoarse;
            }
        }
    }    
    
    public XYCoarse createXYC(int xCoarse, int yCoarse)
    {
        try
        {
            return xycArray[xCoarse+4][yCoarse+2];
        }
        catch (ArrayIndexOutOfBoundsException e)
        {
            Output.print(Output.ERROR, "ERROR: XYCManager.createXYC() called on "
                + xCoarse + "," + yCoarse + " -- out of bounds");
            throw new RuntimeException("ERROR: XYCManager.createXYC() called on "
                + xCoarse + "," + yCoarse + " -- out of bounds");    
            //return xycArray[xCoarseBoundary+4][yCoarseBoundary+1];
        }
    }
    
    public static XYCManager getInstance()
    {
        if (xycManager == null) 
        {
            Output.print(Output.ERROR, "ERROR: XYCManager not initialized");
        }
        return xycManager;
    }
    
    public static void initialize(int xCoarseBoundary, int yCoarseBoundary)
    {
        xycManager = new XYCManager(xCoarseBoundary, yCoarseBoundary);
    }
    
    public static boolean isInitialized()
    {
        return (xycManager != null);
    }
        
} 